@@ -74,6 +74,11 @@ static dc_descriptor_t *getDeviceType(QString btName)
 		product = "i770R";
 	}
 
+	if (btName.contains(QRegularExpression("^ER\\d{6}$"))) {
+		vendor = "Oceanic";
+		product = "Pro Plus X";
+	}
+
 	if (!vendor.isEmpty() && !product.isEmpty())
 		return descriptorLookup.value(vendor + product);
 
@@ -97,10 +102,11 @@ BTDiscovery::BTDiscovery(QObject*) : m_btValid(false),
 void BTDiscovery::BTDiscoveryReDiscover()
 {
 #if !defined(Q_OS_IOS)
+	qDebug() << "BTDiscoveryReDiscover: localBtDevice.isValid()" << localBtDevice.isValid();
 	if (localBtDevice.isValid() &&
-	    localBtDevice.hostMode() == QBluetoothLocalDevice::HostConnectable) {
+	    localBtDevice.hostMode() != QBluetoothLocalDevice::HostPoweredOff) {
 		btPairedDevices.clear();
-		qDebug() <<  "localDevice " + localBtDevice.name() + " is valid, starting discovery";
+		qDebug() <<  "BTDiscoveryReDiscover: localDevice " + localBtDevice.name() + " is powered on, starting discovery";
 #else
 	// for iOS we can't use the localBtDevice as iOS is BLE only
 	// we need to find some other way to test if Bluetooth is enabled, though
@@ -310,6 +316,12 @@ bool BTDiscovery::checkException(const char* method, const QAndroidJniObject *ob
 
 void BTDiscovery::discoverAddress(QString address)
 {
+	// if we have a discovery agent, check if we know about the address and if not
+	// make sure we are looking for it
+	// (if we don't have a discoveryAgent then likely BT is off or something else went wrong)
+	if (!discoveryAgent)
+		return;
+
 	// let's make sure there is no device name mixed in with the address
 	QString btAddress;
 	btAddress = extractBluetoothAddress(address);
