@@ -8,7 +8,8 @@
  * dive_trip_t *dive_trip_list;
  * unsigned int amount_selected;
  * void dump_selection(void)
- * void get_dive_gas(struct dive *dive, int *o2_p, int *he_p, int *o2low_p)
+ * void get_dive_gas(const struct dive *dive, int *o2_p, int *he_p, int *o2low_p)
+ * char *get_dive_gas_string(const struct dive *dive)
  * int total_weight(const struct dive *dive)
  * int get_divenr(const struct dive *dive)
  * int get_divesite_idx(const struct dive_site *ds)
@@ -16,20 +17,34 @@
  * void update_cylinder_related_info(struct dive *dive)
  * void dump_trip_list(void)
  * void insert_trip(dive_trip_t **dive_trip_p)
+ * void insert_trip_dont_merge(dive_trip_t *dive_trip_p)
+ * void unregister_trip(dive_trip_t *trip)
+ * void free_trip(dive_trip_t *trip)
  * void remove_dive_from_trip(struct dive *dive)
+ * void remove_dive_from_trip(struct dive *dive, bool was_autogen)
  * void add_dive_to_trip(struct dive *dive, dive_trip_t *trip)
  * dive_trip_t *create_and_hookup_trip_from_dive(struct dive *dive)
+ * dive_trip_t *get_dives_to_autogroup(int start, int *from, int *to, bool *allocated)
+ * dive_trip_t *get_trip_for_new_dive(struct dive *new_dive, bool *allocated)
  * void autogroup_dives(void)
+ * void combine_trips(struct dive_trip *trip_a, struct dive_trip *trip_b)
+ * dive_trip_t *combine_trips_create(struct dive_trip *trip_a, struct dive_trip *trip_b)
+ * struct dive *unregister_dive(int idx)
  * void delete_single_dive(int idx)
  * void add_single_dive(int idx, struct dive *dive)
- * void merge_two_dives(struct dive *a, struct dive *b)
- * void select_dive(int idx)
- * void deselect_dive(int idx)
+ * struct dive *merge_two_dives(struct dive *a, struct dive *b)
+ * void select_dive(struct dive *dive)
+ * void deselect_dive(struct dive *dive)
  * void mark_divelist_changed(int changed)
  * int unsaved_changes()
  * void remove_autogen_trips()
+ * bool dive_less_than(const struct dive *a, const struct dive *b)
  * void sort_table(struct dive_table *table)
  * bool is_trip_before_after(const struct dive *dive, bool before)
+ * void delete_dive_from_table(struct dive_table *table, int idx)
+ * int find_next_visible_dive(timestamp_t when);
+ * void clear_dive_file_data()
+ * void clear_table(struct dive_table *table)
  */
 #include <unistd.h>
 #include <stdio.h>
@@ -90,14 +105,14 @@ void set_autogroup(bool value)
  *  - Nitrox trumps air (even if hypoxic)
  * These are the same rules as the inter-dive sorting rules.
  */
-void get_dive_gas(struct dive *dive, int *o2_p, int *he_p, int *o2max_p)
+void get_dive_gas(const struct dive *dive, int *o2_p, int *he_p, int *o2max_p)
 {
 	int i;
 	int maxo2 = -1, maxhe = -1, mino2 = 1000;
 
 
 	for (i = 0; i < MAX_CYLINDERS; i++) {
-		cylinder_t *cyl = dive->cylinder + i;
+		const cylinder_t *cyl = dive->cylinder + i;
 		int o2 = get_o2(cyl->gasmix);
 		int he = get_he(cyl->gasmix);
 
@@ -158,7 +173,7 @@ static int calculate_otu(const struct dive *dive)
 		if (sample->setpoint.mbar) {
 			po2 = sample->setpoint.mbar;
 		} else {
-			int o2 = active_o2(dive, dc, sample->time);
+			int o2 = active_o2(dive, dc, psample->time);
 			po2 = lrint(o2 * depth_to_atm(sample->depth.mm, dive));
 		}
 		if (po2 >= 500)
@@ -644,7 +659,7 @@ void update_cylinder_related_info(struct dive *dive)
 #define UTF8_ELLIPSIS "\xE2\x80\xA6"
 
 /* callers needs to free the string */
-char *get_dive_gas_string(struct dive *dive)
+char *get_dive_gas_string(const struct dive *dive)
 {
 	int o2, he, o2max;
 	char *buffer = malloc(MAX_GAS_STRING);
@@ -731,20 +746,35 @@ void insert_trip(dive_trip_t **dive_trip_p)
 #endif
 }
 
-/* create a copy of a dive trip, but don't add any dives. */
-dive_trip_t *clone_empty_trip(dive_trip_t *trip)
+/* same as insert_trip, but don't merge trips with the same date.
+ * this is cruical for the merge undo-command, because there we
+ * add a new trip with the same date and then remove the old one. */
+void insert_trip_dont_merge(dive_trip_t *dive_trip)
 {
-	dive_trip_t *copy = malloc(sizeof(struct dive_trip));
-	*copy = *trip;
-	copy->location = copy_string(trip->location);
-	copy->notes = copy_string(trip->notes);
-	copy->nrdives = 0;
-	copy->next = NULL;
-	copy->dives = NULL;
-	return copy;
+	dive_trip_t **p = &dive_trip_list;
+	dive_trip_t *trip;
+
+	/* Walk the dive trip list looking for the right location.. */
+	while ((trip = *p) != NULL && trip->when < dive_trip->when)
+		p = &trip->next;
+
+	dive_trip->next = trip;
+	*p = dive_trip;
 }
 
-static void delete_trip(dive_trip_t *trip)
+/* free resources associated with a trip structure */
+void free_trip(dive_trip_t *trip)
+{
+	if (trip) {
+		free(trip->location);
+		free(trip->notes);
+		free(trip);
+	}
+}
+
+/* remove trip from the trip-list, but don't free its memory.
+ * caller takes ownership of the trip. */
+void unregister_trip(dive_trip_t *trip)
 {
 	dive_trip_t **p, *tmp;
 
@@ -759,11 +789,12 @@ static void delete_trip(dive_trip_t *trip)
 		}
 		p = &tmp->next;
 	}
+}
 
-	/* .. and free it */
-	free(trip->location);
-	free(trip->notes);
-	free(trip);
+static void delete_trip(dive_trip_t *trip)
+{
+	unregister_trip(trip);
+	free_trip(trip);
 }
 
 void find_new_trip_start_time(dive_trip_t *trip)
@@ -816,13 +847,17 @@ struct dive *last_selected_dive()
 	return ret;
 }
 
-void remove_dive_from_trip(struct dive *dive, short was_autogen)
+/* remove a dive from the trip it's associated to, but don't delete the
+ * trip if this was the last dive in the trip. the caller is responsible
+ * for removing the trip, if the trip->nrdives went to 0.
+ */
+struct dive_trip *unregister_dive_from_trip(struct dive *dive, short was_autogen)
 {
 	struct dive *next, **pprev;
 	dive_trip_t *trip = dive->divetrip;
 
 	if (!trip)
-		return;
+		return NULL;
 
 	/* Remove the dive from the trip's list of dives */
 	next = dive->next;
@@ -837,10 +872,17 @@ void remove_dive_from_trip(struct dive *dive, short was_autogen)
 	else
 		dive->tripflag = NO_TRIP;
 	assert(trip->nrdives > 0);
-	if (!--trip->nrdives)
-		delete_trip(trip);
-	else if (trip->when == dive->when)
+	--trip->nrdives;
+	if (trip->nrdives > 0 && trip->when == dive->when)
 		find_new_trip_start_time(trip);
+	return trip;
+}
+
+void remove_dive_from_trip(struct dive *dive, short was_autogen)
+{
+	struct dive_trip *trip = unregister_dive_from_trip(dive, was_autogen);
+	if (trip && trip->nrdives == 0)
+		delete_trip(trip);
 }
 
 void add_dive_to_trip(struct dive *dive, dive_trip_t *trip)
@@ -849,6 +891,7 @@ void add_dive_to_trip(struct dive *dive, dive_trip_t *trip)
 		return;
 	remove_dive_from_trip(dive, false);
 	trip->nrdives++;
+	trip->showndives++;
 	dive->divetrip = trip;
 	dive->tripflag = ASSIGNED_TRIP;
 
@@ -863,12 +906,27 @@ void add_dive_to_trip(struct dive *dive, dive_trip_t *trip)
 		trip->when = dive->when;
 }
 
+dive_trip_t *alloc_trip(void)
+{
+	return calloc(1, sizeof(dive_trip_t));
+}
+
+dive_trip_t *create_trip_from_dive(struct dive *dive)
+{
+	dive_trip_t *trip;
+
+	trip = alloc_trip();
+	trip->when = dive->when;
+	trip->location = copy_string(get_dive_location(dive));
+
+	return trip;
+}
+
 dive_trip_t *create_and_hookup_trip_from_dive(struct dive *dive)
 {
-	dive_trip_t *dive_trip = calloc(1, sizeof(dive_trip_t));
+	dive_trip_t *dive_trip = alloc_trip();
 
-	dive_trip->when = dive->when;
-	dive_trip->location = copy_string(get_dive_location(dive));
+	dive_trip = create_trip_from_dive(dive);
 	insert_trip(&dive_trip);
 
 	dive->tripflag = IN_TRIP;
@@ -877,14 +935,54 @@ dive_trip_t *create_and_hookup_trip_from_dive(struct dive *dive)
 }
 
 /*
- * Walk the dives from the oldest dive, and see if we can autogroup them
+ * Find a trip a new dive should be autogrouped with. If no such trips
+ * exist, allocate a new trip. The bool "*allocated" is set to true
+ * if a new trip was allocated.
  */
-void autogroup_dives(void)
+dive_trip_t *get_trip_for_new_dive(struct dive *new_dive, bool *allocated)
+{
+	struct dive *d;
+	dive_trip_t *trip;
+	int i;
+
+	/* Find dive that is within TRIP_THRESHOLD of current dive */
+	for_each_dive(i, d) {
+		/* Check if we're past the range of possible dives */
+		if (d->when >= new_dive->when + TRIP_THRESHOLD)
+			break;
+
+		if (d->when + TRIP_THRESHOLD >= new_dive->when && d->divetrip) {
+			/* Found a dive with trip in the range */
+			*allocated = false;
+			return d->divetrip;
+		}
+	}
+
+	/* Didn't find a trip -> allocate a new one */
+	trip = create_trip_from_dive(new_dive);
+	trip->autogen = true;
+	*allocated = true;
+	return trip;
+}
+
+/*
+ * Collect dives for auto-grouping. Pass in first dive which should be checked.
+ * Returns range of dives that should be autogrouped and trip it should be
+ * associated to. If the returned trip was newly allocated, the last bool
+ * is set to true. Caller still has to register it in the system. Note
+ * whereas this looks complicated - it is needed by the undo-system, which
+ * manually injects the new trips. If there are no dives to be autogrouped,
+ * return NULL.
+ */
+dive_trip_t *get_dives_to_autogroup(int start, int *from, int *to, bool *allocated)
 {
 	int i;
 	struct dive *dive, *lastdive = NULL;
 
-	for_each_dive(i, dive) {
+	/* Find first dive that should be merged and remember any previous
+	 * dive that could be merged into.
+	 */
+	for (i = start; (dive = get_dive(i)) != NULL; i++) {
 		dive_trip_t *trip;
 
 		if (dive->divetrip) {
@@ -897,48 +995,107 @@ void autogroup_dives(void)
 			continue;
 		}
 
-		/* Do we have a trip we can combine this into? */
-		if (lastdive && dive->when < lastdive->when + TRIP_THRESHOLD) {
-			dive_trip_t *trip = lastdive->divetrip;
-			add_dive_to_trip(dive, trip);
+		/* We found a dive, let's see if we have to allocate a new trip */
+		if (!lastdive || dive->when >= lastdive->when + TRIP_THRESHOLD) {
+			/* allocate new trip */
+			trip = create_trip_from_dive(dive);
+			trip->autogen = true;
+			*allocated = true;
+		} else {
+			/* use trip of previous dive */
+			trip = lastdive->divetrip;
+			*allocated = false;
+		}
+
+		// Now, find all dives that will be added to this trip
+		lastdive = dive;
+		*from = i;
+		for (*to = *from + 1; (dive = get_dive(*to)) != NULL; (*to)++) {
+			if (dive->divetrip || !DIVE_NEEDS_TRIP(dive) ||
+			    dive->when >= lastdive->when + TRIP_THRESHOLD)
+				break;
 			if (get_dive_location(dive) && !trip->location)
 				trip->location = copy_string(get_dive_location(dive));
 			lastdive = dive;
-			continue;
 		}
-
-		lastdive = dive;
-		trip = create_and_hookup_trip_from_dive(dive);
-		trip->autogen = 1;
+		return trip;
 	}
 
+	/* Did not find anyhting - mark as end */
+	return NULL;
+}
+
+/*
+ * Walk the dives from the oldest dive, and see if we can autogroup them.
+ * But only do this when the user selected autogrouping.
+ */
+void autogroup_dives(void)
+{
+	int from, to;
+	dive_trip_t *trip;
+	int i, j;
+	bool alloc;
+
+	if (!autogroup)
+		return;
+
+	for(i = 0; (trip = get_dives_to_autogroup(i, &from, &to, &alloc)) != NULL; i = to) {
+		/* If this was newly allocated, add trip to list */
+		if (alloc)
+			insert_trip(&trip);
+		for (j = from; j < to; ++j)
+			add_dive_to_trip(get_dive(j), trip);
+	}
 #ifdef DEBUG_TRIP
 	dump_trip_list();
 #endif
 }
 
+static void unregister_dive_from_table(struct dive_table *table, int idx)
+{
+	int i;
+	for (i = idx; i < table->nr - 1; i++)
+		table->dives[i] = table->dives[i + 1];
+	table->dives[--table->nr] = NULL;
+}
+
+/* Remove a dive from a dive table. This assumes that the
+ * dive was already removed from any trip and deselected.
+ * It simply shrinks the table and frees the trip */
+void delete_dive_from_table(struct dive_table *table, int idx)
+{
+	free_dive(table->dives[idx]);
+	unregister_dive_from_table(table, idx);
+}
+
+/* this removes a dive from the dive table and trip-list but doesn't
+ * free the resources associated with the dive. It returns a pointer
+ * to the unregistered dive. The returned dive has the selection-
+ * and hidden-flags cleared. */
+struct dive *unregister_dive(int idx)
+{
+	struct dive *dive = get_dive(idx);
+	if (!dive)
+		return NULL; /* this should never happen */
+	remove_dive_from_trip(dive, false);
+	unregister_dive_from_table(&dive_table, idx);
+	if (dive->selected)
+		amount_selected--;
+	dive->selected = false;
+	return dive;
+}
+
 /* this implements the mechanics of removing the dive from the table,
  * but doesn't deal with updating dive trips, etc */
 void delete_single_dive(int idx)
 {
-	int i;
 	struct dive *dive = get_dive(idx);
 	if (!dive)
 		return; /* this should never happen */
-	remove_dive_from_trip(dive, false);
 	if (dive->selected)
-		deselect_dive(idx);
-	for (i = idx; i < dive_table.nr - 1; i++)
-		dive_table.dives[i] = dive_table.dives[i + 1];
-	dive_table.dives[--dive_table.nr] = NULL;
-	/* free all allocations */
-	free(dive->dc.sample);
-	free((void *)dive->notes);
-	free((void *)dive->divemaster);
-	free((void *)dive->buddy);
-	free((void *)dive->suit);
-	taglist_free(dive->tag_list);
-	free(dive);
+		deselect_dive(dive);
+	dive = unregister_dive(idx);
+	free_dive(dive);
 }
 
 struct dive **grow_dive_table(struct dive_table *table)
@@ -957,25 +1114,30 @@ struct dive **grow_dive_table(struct dive_table *table)
 	return dives;
 }
 
+/* get the index where we want to insert the dive so that everything stays
+ * ordered reverse-chronologically */
+int dive_get_insertion_index(struct dive *dive)
+{
+	/* we might want to use binary search here */
+	for (int i = 0; i < dive_table.nr; i++) {
+		if (dive->when <= dive_table.dives[i]->when)
+			return i;
+	}
+	return dive_table.nr;
+}
+
+/* add a dive at the given index. if the index is negative, the dive will
+ * be added according to reverse chronological order */
 void add_single_dive(int idx, struct dive *dive)
 {
 	int i;
+	if (idx < 0)
+		idx = dive_get_insertion_index(dive);
 	grow_dive_table(&dive_table);
 	dive_table.nr++;
 	if (dive->selected)
 		amount_selected++;
 
-	if (idx < 0) {
-		// convert an idx of -1 so we do insert-in-chronological-order
-		idx = dive_table.nr - 1;
-		for (int i = 0; i < dive_table.nr - 1; i++) {
-			if (dive->when <= dive_table.dives[i]->when) {
-				idx = i;
-				break;
-			}
-		}
-	}
-
 	for (i = idx; i < dive_table.nr; i++) {
 		struct dive *tmp = dive_table.dives[i];
 		dive_table.dives[i] = dive;
@@ -1026,7 +1188,7 @@ struct dive *merge_two_dives(struct dive *a, struct dive *b)
 	if (i < 0 || j < 0)
 		// something is wrong with those dives. Bail
 		return NULL;
-	res = merge_dives(a, b, b->when - a->when, false);
+	res = merge_dives(a, b, b->when - a->when, false, NULL);
 	if (!res)
 		return NULL;
 
@@ -1088,42 +1250,44 @@ struct dive *merge_two_dives(struct dive *a, struct dive *b)
 	return res;
 }
 
-void select_dive(int idx)
+void select_dive(struct dive *dive)
 {
-	struct dive *dive = get_dive(idx);
-	if (dive) {
-		/* never select an invalid dive that isn't displayed */
-		if (!dive->selected) {
-			dive->selected = 1;
-			amount_selected++;
-		}
-		selected_dive = idx;
+	if (!dive)
+		return;
+	if (!dive->selected) {
+		dive->selected = 1;
+		amount_selected++;
 	}
+	current_dive = dive;
 }
 
-void deselect_dive(int idx)
+void deselect_dive(struct dive *dive)
 {
-	struct dive *dive = get_dive(idx);
+	int idx;
 	if (dive && dive->selected) {
 		dive->selected = 0;
 		if (amount_selected)
 			amount_selected--;
-		if (selected_dive == idx && amount_selected > 0) {
+		if (current_dive == dive && amount_selected > 0) {
 			/* pick a different dive as selected */
+			int selected_dive = idx = get_divenr(dive);
 			while (--selected_dive >= 0) {
 				dive = get_dive(selected_dive);
-				if (dive && dive->selected)
+				if (dive && dive->selected) {
+					current_dive = dive;
 					return;
+				}
 			}
 			selected_dive = idx;
 			while (++selected_dive < dive_table.nr) {
 				dive = get_dive(selected_dive);
-				if (dive && dive->selected)
+				if (dive && dive->selected) {
+					current_dive = dive;
 					return;
+				}
 			}
 		}
-		if (amount_selected == 0)
-			selected_dive = -1;
+		current_dive = NULL;
 	}
 }
 
@@ -1133,7 +1297,7 @@ void deselect_dives_in_trip(struct dive_trip *trip)
 	if (!trip)
 		return;
 	for (dive = trip->dives; dive; dive = dive->next)
-		deselect_dive(get_divenr(dive));
+		deselect_dive(dive);
 }
 
 void select_dives_in_trip(struct dive_trip *trip)
@@ -1143,7 +1307,7 @@ void select_dives_in_trip(struct dive_trip *trip)
 		return;
 	for (dive = trip->dives; dive; dive = dive->next)
 		if (!dive->hidden_by_filter)
-			select_dive(get_divenr(dive));
+			select_dive(dive);
 }
 
 void filter_dive(struct dive *d, bool shown)
@@ -1152,7 +1316,7 @@ void filter_dive(struct dive *d, bool shown)
 		return;
 	d->hidden_by_filter = !shown;
 	if (!shown && d->selected)
-		deselect_dive(get_divenr(d));
+		deselect_dive(d);
 }
 
 
@@ -1175,6 +1339,26 @@ void combine_trips(struct dive_trip *trip_a, struct dive_trip *trip_b)
 		add_dive_to_trip(trip_b->dives, trip_a);
 }
 
+/* Out of two strings, copy the string that is not empty (if any). */
+static char *copy_non_empty_string(const char *a, const char *b)
+{
+	return copy_string(empty_string(b) ? a : b);
+}
+
+/* Combine trips new. This combines two trips, generating a
+ * new trip. To support undo, we have to preserve the old trips. */
+dive_trip_t *combine_trips_create(struct dive_trip *trip_a, struct dive_trip *trip_b)
+{
+	dive_trip_t *trip;
+
+	trip = alloc_trip();
+	trip->when = trip_a->when;
+	trip->location = copy_non_empty_string(trip_a->location, trip_b->location);
+	trip->notes = copy_non_empty_string(trip_a->notes, trip_b->notes);
+
+	return trip;
+}
+
 void mark_divelist_changed(bool changed)
 {
 	if (dive_list_changed == changed)
@@ -1226,16 +1410,16 @@ void remove_autogen_trips()
  * what the numbers should be - in which case you need to do
  * a manual re-numbering.
  */
-static void try_to_renumber(struct dive *last, int preexisting)
+static void try_to_renumber(int preexisting)
 {
 	int i, nr;
+	struct dive *last = get_dive(preexisting - 1);
 
 	/*
-	 * If the new dives aren't all strictly at the end,
-	 * we're going to expect the user to do a manual
-	 * renumbering.
+	 * If there was a last dive, but it didn't have
+	 * a number, give up.
 	 */
-	if (preexisting && get_dive(preexisting - 1) != last)
+	if (last && !last->number)
 		return;
 
 	/*
@@ -1273,40 +1457,18 @@ void process_loaded_dives()
 	sort_table(&dive_table);
 }
 
-void process_imported_dives(bool prefer_imported)
+/*
+ * Merge subsequent dives in a table, if mergeable. This assumes
+ * that the dives are neither selected, not part of a trip, as
+ * is the case of freshly imported dives.
+ */
+static void merge_imported_dives(struct dive_table *table)
 {
 	int i;
-	int preexisting = dive_table.preexisting;
-	struct dive *last;
-
-	/* If no dives were imported, don't bother doing anything */
-	if (preexisting >= dive_table.nr)
-		return;
-
-	/* check if we need a nickname for the divecomputer for newly downloaded dives;
-	 * since we know they all came from the same divecomputer we just check for the
-	 * first one */
-	if (dive_table.dives[preexisting]->downloaded)
-		set_dc_nickname(dive_table.dives[preexisting]);
-	else
-		/* they aren't downloaded, so record / check all new ones */
-		for (i = preexisting; i < dive_table.nr; i++)
-			set_dc_nickname(dive_table.dives[i]);
-
-	for (i = preexisting; i < dive_table.nr; i++)
-		dive_table.dives[i]->downloaded = true;
-
-	/* This does the right thing for -1: NULL */
-	last = get_dive(preexisting - 1);
-
-	sort_table(&dive_table);
-
-	for (i = 1; i < dive_table.nr; i++) {
-		struct dive **pp = &dive_table.dives[i - 1];
-		struct dive *prev = pp[0];
-		struct dive *dive = pp[1];
+	for (i = 1; i < table->nr; i++) {
+		struct dive *prev = table->dives[i - 1];
+		struct dive *dive = table->dives[i];
 		struct dive *merged;
-		int id;
 
 		/* only try to merge overlapping dives - or if one of the dives has
 		 * zero duration (that might be a gps marker from the webservice) */
@@ -1314,39 +1476,162 @@ void process_imported_dives(bool prefer_imported)
 		    dive_endtime(prev) < dive->when)
 			continue;
 
-		merged = try_to_merge(prev, dive, prefer_imported);
+		merged = try_to_merge(prev, dive, false);
 		if (!merged)
 			continue;
 
-		// remember the earlier dive's id
-		id = prev->id;
-
-		/* careful - we might free the dive that last points to. Oops... */
-		if (last == prev || last == dive)
-			last = merged;
+		/* Overwrite the first of the two dives and remove the second */
+		free_dive(prev);
+		table->dives[i - 1] = merged;
+		delete_dive_from_table(table, i);
 
 		/* Redo the new 'i'th dive */
 		i--;
-		add_single_dive(i, merged);
-		delete_single_dive(i + 1);
-		delete_single_dive(i + 1);
-		// keep the id or the first dive for the merged dive
-		merged->id = id;
+	}
+}
+
+/*
+ * Try to merge a new dive into the dive at position idx. Return
+ * true on success. On success, the dive to add and the old dive
+ * will be deleted. On failure, they are untouched.
+ * If "prefer_imported" is true, use data of the new dive.
+ */
+static bool try_to_merge_into(struct dive *dive_to_add, int idx, bool prefer_imported)
+{
+	struct dive *old_dive = dive_table.dives[idx];
+	struct dive_trip *trip = old_dive->divetrip;
+	struct dive *merged = try_to_merge(old_dive, dive_to_add, prefer_imported);
+	if (!merged)
+		return false;
+
+	merged->id = old_dive->id;
+	merged->selected = old_dive->selected;
+	dive_table.dives[idx] = merged;
+	if (trip) {
+		remove_dive_from_trip(old_dive, false);
+		add_dive_to_trip(merged, trip);
+	}
+	free_dive(old_dive);
+	remove_dive_from_trip(dive_to_add, false);
+	free_dive(dive_to_add);
+
+	return true;
+}
+
+/*
+ * Add imported dive to global dive table. Overlapping dives will
+ * be merged if possible. If prefer_imported is true, data of the
+ * new dives are prioritized in such a case.
+ * If downloaded is true, only the divecomputer of the first dive
+ * will be considered, as it is assumed that all dives come from
+ * the same computer.
+ * Note: the dives in import_table are consumed! On return import_table
+ * has size 0.
+ */
+void process_imported_dives(struct dive_table *import_table, bool prefer_imported, bool downloaded)
+{
+	int i, j;
+	int preexisting;
+	bool sequence_changed = false;
+
+	/* If no dives were imported, don't bother doing anything */
+	if (!import_table->nr)
+		return;
+
+	/* check if we need a nickname for the divecomputer for newly downloaded dives;
+	 * since we know they all came from the same divecomputer we just check for the
+	 * first one */
+	if (downloaded)
+		set_dc_nickname(import_table->dives[0]);
+	else
+		/* they aren't downloaded, so record / check all new ones */
+		for (i = 0; i < import_table->nr; i++)
+			set_dc_nickname(import_table->dives[i]);
+
+	/* Sort the table of dives to be imported and combine mergable dives */
+	sort_table(import_table);
+	merge_imported_dives(import_table);
+
+	/* Merge newly imported dives into the dive table.
+	 * Since both lists (old and new) are sorted, we can step
+	 * through them concurrently and locate the insertions points.
+	 * Once found, check if the new dive can be merged in the
+	 * previous or next dive.
+	 * Note that this doesn't consider pathological cases such as:
+	 *  - New dive "connects" two old dives (turn three into one).
+	 *  - New dive can not be merged into adjacent but some further dive.
+	 */
+	j = 0; /* Index in old dives */
+	preexisting = dive_table.nr; /* Remember old size for renumbering */
+	for (i = 0; i < import_table->nr; i++) {
+		struct dive *dive_to_add = import_table->dives[i];
+
+		/* Find insertion point. */
+		while (j < dive_table.nr && dive_table.dives[j]->when < dive_to_add->when)
+			j++;
+
+		/* Try to merge into previous dive. */
+		if (j > 0 && dive_endtime(dive_table.dives[j - 1]) > dive_to_add->when) {
+			if (try_to_merge_into(dive_to_add, j - 1, prefer_imported))
+				continue;
+		}
+
+		/* That didn't merge into the previous dive. If we're
+		 * at the end of the dive table, quit the loop and add
+		 * all new dives at the end. */
+		if (j >= dive_table.nr)
+			break;
+
+		/* Try to merge into next dive. */
+		if (dive_endtime(dive_to_add) > dive_table.dives[j]->when) {
+			if (try_to_merge_into(dive_to_add, j, prefer_imported))
+				continue;
+		}
+
+		/* We couldnt merge dives, add at the given position. */
+		add_single_dive(j, dive_to_add);
+		j++;
+		sequence_changed = true;
 	}
 
-	/* make sure no dives are still marked as downloaded */
-	for (i = 1; i < dive_table.nr; i++)
-		dive_table.dives[i]->downloaded = false;
+	/* If there are still dives to add, add them at the end of the dive table. */
+	for ( ; i <  import_table->nr; i++)
+		add_single_dive(dive_table.nr, import_table->dives[i]);
 
-	/* If there are dives in the table, are they numbered */
-	if (!last || last->number)
-		try_to_renumber(last, preexisting);
+	/* we took care of all dives, clean up the import table */
+	import_table->nr = 0;
 
+	/* If the sequence wasn't changed, renumber */
+	if (!sequence_changed)
+		try_to_renumber(preexisting);
+
+	/* We might have deleted the old selected dive.
+	 * Choose the newest dive as selected (if any) */
+	current_dive = dive_table.nr > 0 ? dive_table.dives[dive_table.nr - 1] : NULL;
 	mark_divelist_changed(true);
 }
 
+/* return the number a dive gets when inserted at the given index.
+ * this function is supposed to be called *before* a dive was added.
+ * this returns:
+ * 	- 1 for an empty log
+ * 	- last_nr+1 for addition at end of log (if last dive had a number)
+ * 	- 0 for all other cases
+ */
+int get_dive_nr_at_idx(int idx)
+{
+	if (dive_table.nr == 0)
+		return 1;
+	if (idx >= dive_table.nr) {
+		struct dive *last_dive = get_dive(dive_table.nr - 1);
+		return last_dive->number ? last_dive->number + 1 : 0;
+	}
+	return 0;
+}
+
 void set_dive_nr_for_current_dive()
 {
+	int selected_dive = get_divenr(current_dive);
 	if (dive_table.nr == 1)
 		current_dive->number = 1;
 	else if (selected_dive == dive_table.nr - 1 && get_dive(dive_table.nr - 2)->number)
@@ -1402,28 +1687,120 @@ void clear_dive_file_data()
 	while (dive_table.nr)
 		delete_single_dive(0);
 	while (dive_site_table.nr)
-		delete_dive_site(get_dive_site(0)->uuid);
+		delete_dive_site(get_dive_site(0));
 
 	clear_dive(&displayed_dive);
-	clear_dive_site(&displayed_dive_site);
 
 	reset_min_datafile_version();
 	saved_git_id = "";
 }
 
-static int sortfn(const void *_a, const void *_b)
+/*
+ * Clear a dive_table
+ */
+void clear_table(struct dive_table *table)
 {
-	const struct dive *a = (const struct dive *)*(void **)_a;
-	const struct dive *b = (const struct dive *)*(void **)_b;
+	for (int i = 0; i < table->nr; i++)
+		free_dive(table->dives[i]);
+	table->nr = 0;
+}
 
+/* This function defines the sort ordering of dives. The core
+ * and the UI models should use the same sort function, which
+ * should be stable. This is not crucial at the moment, as the
+ * indices in core and UI are independent, but ultimately we
+ * probably want to unify the models.
+ * After editing a key used in this sort-function, the order of
+ * the dives must be re-astablished.
+ * Currently, this does a lexicographic sort on the (start-time, id)
+ * tuple. "id" is a stable, strictly increasing unique number, that
+ * is handed out when a dive is added to the system.
+ * We might also consider sorting by end-time and other criteria,
+ * but see the caveat above (editing means rearrangement of the dives).
+ */
+static int comp_dives(const struct dive *a, const struct dive *b)
+{
 	if (a->when < b->when)
 		return -1;
 	if (a->when > b->when)
 		return 1;
-	return 0;
+	if (a->id < b->id)
+		return -1;
+	if (a->id > b->id)
+		return 1;
+	return 0; /* this should not happen for a != b */
+}
+
+bool dive_less_than(const struct dive *a, const struct dive *b)
+{
+	return comp_dives(a, b) < 0;
+}
+
+static int sortfn(const void *_a, const void *_b)
+{
+	const struct dive *a = (const struct dive *)*(const void **)_a;
+	const struct dive *b = (const struct dive *)*(const void **)_b;
+	return comp_dives(a, b);
 }
 
 void sort_table(struct dive_table *table)
 {
 	qsort(table->dives, table->nr, sizeof(struct dive *), sortfn);
 }
+
+/*
+ * Calculate surface interval for dive starting at "when". Currently, we
+ * might display dives which are not yet in the divelist, therefore the
+ * input parameter is a timestamp.
+ * If the given dive starts during a different dive, the surface interval
+ * is 0. If we can't determine a surface interval (first dive), <0 is
+ * returned. This does *not* consider pathological cases such as dives
+ * that happened inside other dives. The interval will always be calculated
+ * with respect to the dive that started previously.
+ */
+timestamp_t get_surface_interval(timestamp_t when)
+{
+	int i;
+	timestamp_t prev_end;
+
+	/* find previous dive. might want to use a binary search. */
+	for (i = dive_table.nr - 1; i >= 0; --i) {
+		if (dive_table.dives[i]->when < when)
+			break;
+	}
+	if (i < 0)
+		return -1;
+
+	prev_end = dive_endtime(dive_table.dives[i]);
+	if (prev_end > when)
+		return 0;
+	return when - prev_end;
+}
+
+/* Find visible dive close to given date. First search towards older,
+ * then newer dives. */
+struct dive *find_next_visible_dive(timestamp_t when)
+{
+	int i, j;
+
+	if (!dive_table.nr)
+		return NULL;
+
+	/* we might want to use binary search here */
+	for (i = 0; i < dive_table.nr; i++) {
+		if (when <= get_dive(i)->when)
+			break;
+	}
+
+	for (j = i - 1; j > 0; j--) {
+		if (!get_dive(j)->hidden_by_filter)
+			return get_dive(j);
+	}
+
+	for (j = i; j < dive_table.nr; j++) {
+		if (!get_dive(j)->hidden_by_filter)
+			return get_dive(j);
+	}
+
+	return NULL;
+}
