@@ -30,7 +30,7 @@ typedef struct dc_user_device_t
 {
 	dc_descriptor_t *descriptor;
 	const char *vendor, *product, *devname;
-	const char *model;
+	const char *model, *btname;
 	unsigned char *fingerprint;
 	unsigned int fsize, fdiveid;
 	uint32_t libdc_firmware, libdc_serial;
