@@ -63,7 +63,6 @@ typedef struct {
 	const char *product;
 	const char *device;
 	const char *device_name;
-	int download_mode;
 } dive_computer_prefs_t;
 
 enum unit_system_values {
@@ -87,6 +86,7 @@ struct preferences {
 	int animation_speed;
 
 	// ********** CloudStorage **********
+	bool       cloud_auto_sync;
 	const char *cloud_base_url;
 	const char *cloud_git_url;
 	const char *cloud_storage_email;
@@ -126,6 +126,8 @@ struct preferences {
 	int         o2consumption; // ml per min
 	int         pscr_ratio; // dump ratio times 1000
 	bool        use_default_file;
+	bool        filterFullTextNotes; // mobile only - include notes information in full text searh
+	bool        filterCaseSensitive; // mobile only - make fltering case sensitive
 
 	// ********** Geocoding **********
 	geocoding_prefs_t geocoding;
