@@ -81,6 +81,11 @@ int DiveObjectHelper::id() const
 	return m_dive->id;
 }
 
+struct dive *DiveObjectHelper::getDive() const
+{
+	return m_dive;
+}
+
 QString DiveObjectHelper::date() const
 {
 	QDateTime localTime = QDateTime::fromMSecsSinceEpoch(1000*m_dive->when, Qt::UTC);
@@ -107,8 +112,8 @@ QString DiveObjectHelper::location() const
 
 QString DiveObjectHelper::gps() const
 {
-	struct dive_site *ds = get_dive_site_by_uuid(m_dive->dive_site_uuid);
-	return ds ? QString(printGPSCoords(ds->latitude.udeg, ds->longitude.udeg)) : QString();
+	struct dive_site *ds = m_dive->dive_site;
+	return ds ? QString(printGPSCoords(&ds->location)) : QString();
 }
 
 QString DiveObjectHelper::gps_decimal() const
@@ -122,9 +127,9 @@ QString DiveObjectHelper::gps_decimal() const
 	return val;
 }
 
-QVariant DiveObjectHelper::dive_site_uuid() const
+QVariant DiveObjectHelper::dive_site() const
 {
-	return QVariant::fromValue(m_dive->dive_site_uuid);
+	return QVariant::fromValue(m_dive->dive_site);
 }
 
 QString DiveObjectHelper::duration() const
@@ -336,12 +341,12 @@ QString DiveObjectHelper::tripMeta() const
 	QString ret = EMPTY_DIVE_STRING;
 	struct dive_trip *dt = m_dive->divetrip;
 	if (dt) {
-		QString numDives = tr("(%n dive(s))", "", dt->nrdives);
+		QString numDives = tr("(%n dive(s))", "", dt->showndives);
 		QString title(dt->location);
 		QDateTime firstTime = QDateTime::fromMSecsSinceEpoch(1000*dt->when, Qt::UTC);
 		QString firstMonth = firstTime.toString("MMM");
 		QString tripDate = QStringLiteral("%1@%2").arg(firstMonth,firstTime.toString("yy"));
-;
+
 		if (title.isEmpty()) {
 			// so use the date range
 			QString firstYear = firstTime.toString("yyyy");
@@ -434,3 +439,14 @@ QStringList DiveObjectHelper::firstGas() const
 	}
 	return gas;
 }
+
+// for a full text search / filter function
+QString DiveObjectHelper::fullText() const
+{
+	return fullTextNoNotes() + ":-:" + notes();
+}
+
+QString DiveObjectHelper::fullTextNoNotes() const
+{
+	return trip() + ":-:" + location() + ":-:" + buddy() + ":-:" + divemaster() + ":-:" + suit() + ":-:" + tags();
+}
