@@ -79,7 +79,7 @@ static uint32_t mindiveid;
 /* Linked list to remember already executed divespot download requests */
 struct divespot_mapping {
 	int divespot_id;
-	uint32_t dive_site_uuid;
+	struct dive_site *dive_site;
 	struct divespot_mapping *next;
 };
 static struct divespot_mapping *divespot_mapping = NULL;
@@ -95,7 +95,7 @@ static void erase_divespot_mapping()
 	divespot_mapping = NULL;
 }
 
-static void add_to_divespot_mapping(int divespot_id, uint32_t dive_site_uuid)
+static void add_to_divespot_mapping(int divespot_id, struct dive_site *ds)
 {
 	struct divespot_mapping *ndm = (struct divespot_mapping*)calloc(1, sizeof(struct divespot_mapping));
 	struct divespot_mapping **pdm = &divespot_mapping;
@@ -105,7 +105,7 @@ static void add_to_divespot_mapping(int divespot_id, uint32_t dive_site_uuid)
 		cdm = cdm->next;
 	
 	ndm->divespot_id = divespot_id;
-	ndm->dive_site_uuid = dive_site_uuid;
+	ndm->dive_site = ds;
 	ndm->next = NULL;
 	if (cdm)
 		cdm->next = ndm;
@@ -124,15 +124,15 @@ static bool is_divespot_mappable(int divespot_id)
 	return false;
 }
 
-static uint32_t get_dive_site_uuid_by_divespot_id(int divespot_id)
+static struct dive_site *get_dive_site_by_divespot_id(int divespot_id)
 {
 	struct divespot_mapping *dm = divespot_mapping;
 	while (dm) {
 		if (dm->divespot_id == divespot_id)
-			return dm->dive_site_uuid;
+			return dm->dive_site;
 		dm = dm->next;
 	}
-	return 0;
+	return NULL;
 }
 
 /* helper function to parse the Uemis data structures */
@@ -197,7 +197,6 @@ static void uemis_get_weight(char *buffer, weightsystem_t *weight, int diveid)
 static struct dive *uemis_start_dive(uint32_t deviceid)
 {
 	struct dive *dive = alloc_dive();
-	dive->downloaded = true;
 	dive->dc.model = strdup("Uemis Zurich");
 	dive->dc.deviceid = deviceid;
 	return dive;
@@ -993,11 +992,12 @@ static bool process_raw_buffer(device_data_t *devdata, uint32_t deviceid, char *
 		} else if (!is_log && dive && !strcmp(tag, "divespot_id")) {
 			int divespot_id = atoi(val);
 			if (divespot_id != -1) {
-				dive->dive_site_uuid = create_dive_site("from Uemis", dive->when);
-				uemis_mark_divelocation(dive->dc.diveid, divespot_id, dive->dive_site_uuid);
+				struct dive_site *ds = create_dive_site("from Uemis", dive->when);
+				dive->dive_site = ds;
+				uemis_mark_divelocation(dive->dc.diveid, divespot_id, ds);
 			}
 #if UEMIS_DEBUG & 2
-			fprintf(debugfile, "Created divesite %d for diveid : %d\n", dive->dive_site_uuid, dive->dc.diveid);
+			fprintf(debugfile, "Created divesite %d for diveid : %d\n", dive->dive_site->uuid, dive->dc.diveid);
 #endif
 		} else if (dive) {
 			parse_tag(dive, tag, val);
@@ -1029,12 +1029,11 @@ static bool process_raw_buffer(device_data_t *devdata, uint32_t deviceid, char *
 	return true;
 }
 
-static char *uemis_get_divenr(char *deviceidstr, int force)
+static char *uemis_get_divenr(char *deviceidstr, struct dive_table *table, int force)
 {
 	uint32_t deviceid, maxdiveid = 0;
 	int i;
 	char divenr[10];
-	struct dive_table *table;
 	deviceid = atoi(deviceidstr);
 	mindiveid = 0xFFFFFFFF;
 
@@ -1044,11 +1043,11 @@ static char *uemis_get_divenr(char *deviceidstr, int force)
 	 * already have.
 	 *
 	 * Also, if "force_download" is true, do this even if we
-	 * don't have any dives (maxdiveid will remain zero)
+	 * don't have any dives (maxdiveid will remain ~0).
+	 *
+	 * Otherwise, use the global dive table.
 	 */
-	if (force || downloadTable.nr)
-		table = &downloadTable;
-	else
+	if (!force && !table->nr)
 		table = &dive_table;
 
 	for (i = 0; i < table->nr; i++) {
@@ -1148,12 +1147,12 @@ static int get_memory(struct dive_table *td, int checkpoint)
 	return UEMIS_MEM_OK;
 }
 
-/* mark a dive as deleted by setting download to false
- * this will be picked up by some cleaning statement later */
+/* we misuse the hidden_by_filter flag to mark a dive as deleted.
+ * this will be picked up by some cleaning statement later. */
 static void do_delete_dives(struct dive_table *td, int idx)
 {
 	for (int x = idx; x < td->nr; x++)
-		td->dives[x]->downloaded = false;
+		td->dives[x]->hidden_by_filter = true;
 }
 
 static bool load_uemis_divespot(const char *mountpath, int divespot_id)
@@ -1176,34 +1175,35 @@ static bool load_uemis_divespot(const char *mountpath, int divespot_id)
 
 static void get_uemis_divespot(const char *mountpath, int divespot_id, struct dive *dive)
 {
-	struct dive_site *nds = get_dive_site_by_uuid(dive->dive_site_uuid);
+	struct dive_site *nds = dive->dive_site;
 	
 	if (is_divespot_mappable(divespot_id)) {
-		dive->dive_site_uuid = get_dive_site_uuid_by_divespot_id(divespot_id);
+		struct dive_site *ds = get_dive_site_by_divespot_id(divespot_id);
+		dive->dive_site = ds;
 	} else if (nds && nds->name && strstr(nds->name,"from Uemis")) {
 		if (load_uemis_divespot(mountpath, divespot_id)) {
 			/* get the divesite based on the diveid, this should give us
 			* the newly created site
 			*/
-			struct dive_site *ods = NULL;
+			struct dive_site *ods;
 			/* with the divesite name we got from parse_dive, that is called on load_uemis_divespot
 			 * we search all existing divesites if we have one with the same name already. The function
 			 * returns the first found which is luckily not the newly created.
 			 */
-			(void)get_dive_site_uuid_by_name(nds->name, &ods);
+			ods = get_dive_site_by_name(nds->name);
 			if (ods) {
 				/* if the uuid's are the same, the new site is a duplicate and can be deleted */
 				if (nds->uuid != ods->uuid) {
-					delete_dive_site(nds->uuid);
-					dive->dive_site_uuid = ods->uuid;
+					delete_dive_site(nds);
+					dive->dive_site = ods;
 				}
 			}
-			add_to_divespot_mapping(divespot_id, dive->dive_site_uuid);
+			add_to_divespot_mapping(divespot_id, dive->dive_site);
 		} else {
 			/* if we can't load the dive site details, delete the site we
 			* created in process_raw_buffer
 			*/
-			delete_dive_site(dive->dive_site_uuid);
+			delete_dive_site(dive->dive_site);
 		}
 	}
 }
@@ -1266,7 +1266,7 @@ static bool get_matching_dive(int idx, char *newmax, int *uemis_mem_status, devi
 #endif
 						deleted_files++;
 						/* mark this log entry as deleted and cleanup later, otherwise we mess up our array */
-						dive->downloaded = false;
+						dive->hidden_by_filter = true;
 #if UEMIS_DEBUG & 2
 						fprintf(debugfile, "Deleted dive from %s, with id %d from table -- newmax is %s\n", d_time, dive->dc.diveid, newmax);
 #endif
@@ -1350,7 +1350,7 @@ const char *do_uemis_import(device_data_t *data)
 		goto bail;
 
 	param_buff[1] = "notempty";
-	newmax = uemis_get_divenr(deviceid, force_download);
+	newmax = uemis_get_divenr(deviceid, data->download_table, force_download);
 	if (verbose)
 		fprintf(stderr, "Uemis downloader: start looking at dive nr %s\n", newmax);
 
@@ -1472,7 +1472,7 @@ const char *do_uemis_import(device_data_t *data)
 	 * to see if we have to clean some dead bodies from our download table */
 	next_table_index = 0;
 	while (next_table_index < data->download_table->nr) {
-		if (!data->download_table->dives[next_table_index]->downloaded)
+		if (data->download_table->dives[next_table_index]->hidden_by_filter)
 			uemis_delete_dive(data, data->download_table->dives[next_table_index]->dc.diveid);
 		else
 			next_table_index++;
