@@ -20,31 +20,35 @@
 #include <QMessageBox>
 #include <QHeaderView>
 #include "core/qthelper.h"
-#include "desktop-widgets/undocommands.h"
+#include "desktop-widgets/command.h"
 #include "desktop-widgets/divelistview.h"
 #include "qt-models/divepicturemodel.h"
 #include "core/metrics.h"
+#include "core/subsurface-qt/DiveListNotifier.h"
 
-DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelection(false), sortColumn(0),
-	currentOrder(Qt::DescendingOrder), dontEmitDiveChangedSignal(false), selectionSaved(false),
+DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelection(false),
+	currentLayout(DiveTripModel::TREE), dontEmitDiveChangedSignal(false), selectionSaved(false),
 	initialColumnWidths(DiveTripModel::COLUMNS, 50)	// Set up with default length 50
 {
 	setItemDelegate(new DiveListDelegate(this));
 	setUniformRowHeights(true);
 	setItemDelegateForColumn(DiveTripModel::RATING, new StarWidgetsDelegate(this));
-	MultiFilterSortModel *model = MultiFilterSortModel::instance();
-	model->setSortRole(DiveTripModel::SORT_ROLE);
-	model->setFilterKeyColumn(-1); // filter all columns
-	model->setFilterCaseSensitivity(Qt::CaseInsensitive);
-	model->setSourceModel(DiveTripModel::instance());
-	setModel(model);
+	setModel(MultiFilterSortModel::instance());
 
 	setSortingEnabled(false);
 	setContextMenuPolicy(Qt::DefaultContextMenu);
 	setSelectionMode(ExtendedSelection);
 	header()->setContextMenuPolicy(Qt::ActionsContextMenu);
+	connect(DiveTripModel::instance(), &DiveTripModel::selectionChanged, this, &DiveListView::diveSelectionChanged);
+	connect(DiveTripModel::instance(), &DiveTripModel::newCurrentDive, this, &DiveListView::currentDiveChanged);
+
+	// Update selection if all selected dives were hidden by filter
+	connect(MultiFilterSortModel::instance(), &MultiFilterSortModel::filterFinished, this, &DiveListView::filterFinished);
 
 	header()->setStretchLastSection(true);
+	header()->setSortIndicatorShown(true);
+	header()->setSectionsClickable(true);
+	connect(header(), &QHeaderView::sortIndicatorChanged, this, &DiveListView::sortIndicatorChanged);
 
 	installEventFilter(this);
 
@@ -175,16 +179,61 @@ void DiveListView::reset()
 	}
 }
 
+// If items were selected, inform the selection model
+void DiveListView::diveSelectionChanged(const QVector<QModelIndex> &indexes, bool select)
+{
+	MultiFilterSortModel *m = MultiFilterSortModel::instance();
+	QItemSelectionModel *s = selectionModel();
+	auto flags = select ?
+		QItemSelectionModel::Rows | QItemSelectionModel::Select :
+		QItemSelectionModel::Rows | QItemSelectionModel::Deselect;
+	for (const QModelIndex &index: indexes) {
+		// We have to transform the indices into local indices, since
+		// there might be sorting or filtering in effect.
+		QModelIndex localIndex = m->mapFromSource(index);
+
+		// It might be possible that the item is not shown (filter is
+		// in effect). Then we get an invalid index and should ignore
+		// this selection.
+		if (!localIndex.isValid())
+			continue;
+
+		s->select(localIndex, flags);
+
+		// If an item of a not-yet expanded trip is selected, expand the trip.
+		if (select && localIndex.parent().isValid() && !isExpanded(localIndex.parent())) {
+			setAnimated(false);
+			expand(localIndex.parent());
+			setAnimated(true);
+		}
+	}
+}
+
+void DiveListView::currentDiveChanged(QModelIndex index)
+{
+	// Transform the index into a local index, since
+	// there might be sorting or filtering in effect.
+	MultiFilterSortModel *m = MultiFilterSortModel::instance();
+	QModelIndex localIndex = m->mapFromSource(index);
+
+	// Then, set the currently activated row.
+	// Note, we have to use the QItemSelectionModel::Current mode to avoid
+	// changing our selection (in contrast to Qt's documentation, which
+	// instructs to use QItemSelectionModel::NoUpdate, which results in
+	// funny side-effects).
+	selectionModel()->setCurrentIndex(localIndex, QItemSelectionModel::Current);
+}
+
 // If rows are added, check which of these rows is a trip and expand the first column
 void DiveListView::rowsInserted(const QModelIndex &parent, int start, int end)
 {
 	// First, let the QTreeView do its thing.
 	QTreeView::rowsInserted(parent, start, end);
 
-	// Now check for each inserted row whether this is a trip and expand the first column
+	// Check for each inserted row whether this is a trip and expand the first column
+	QAbstractItemModel *m = model();
 	if (parent.isValid()) // Trips don't have a parent
 		return;
-	QAbstractItemModel *m = model();
 	for (int i = start; i <= end; ++i) {
 		if (m->rowCount(m->index(i, 0)) != 0)
 			setFirstColumnSpanned(i, QModelIndex(), true);
@@ -238,7 +287,7 @@ void DiveListView::selectTrip(dive_trip_t *trip)
 	if (!trip)
 		return;
 
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
+	QAbstractItemModel *m = model();
 	QModelIndexList match = m->match(m->index(0, 0), DiveTripModel::TRIP_ROLE, QVariant::fromValue<void *>(trip), 2, Qt::MatchRecursive);
 	QItemSelectionModel::SelectionFlags flags;
 	if (!match.count())
@@ -258,25 +307,23 @@ void DiveListView::selectTrip(dive_trip_t *trip)
 // works as expected
 void DiveListView::clearTripSelection()
 {
-	// we want to make sure no trips are selected
-	disconnect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(selectionChanged(QItemSelection, QItemSelection)));
-	disconnect(selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)), this, SLOT(currentChanged(QModelIndex, QModelIndex)));
+	// This marks the selection change as being internal - ie. we don't process it further.
+	// TODO: This should probably be sold differently.
+	auto marker = diveListNotifier.enterCommand();
 
+	// we want to make sure no trips are selected
 	Q_FOREACH (const QModelIndex &index, selectionModel()->selectedRows()) {
 		dive_trip_t *trip = static_cast<dive_trip_t *>(index.data(DiveTripModel::TRIP_ROLE).value<void *>());
 		if (!trip)
 			continue;
 		selectionModel()->select(index, QItemSelectionModel::Deselect);
 	}
-
-	connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(selectionChanged(QItemSelection, QItemSelection)));
-	connect(selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)), this, SLOT(currentChanged(QModelIndex, QModelIndex)));
 }
 
 void DiveListView::unselectDives()
 {
 	// make sure we don't try to redraw the dives during the selection change
-	selected_dive = -1;
+	current_dive = nullptr;
 	amount_selected = 0;
 	// clear the Qt selection
 	selectionModel()->clearSelection();
@@ -303,17 +350,11 @@ QList<dive_trip_t *> DiveListView::selectedTrips()
 	return ret;
 }
 
-void DiveListView::selectDive(int i, bool scrollto, bool toggle)
+void DiveListView::selectDive(QModelIndex idx, bool scrollto, bool toggle)
 {
-	if (i == -1)
+	if (!idx.isValid())
 		return;
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
-	QModelIndexList match = m->match(m->index(0, 0), DiveTripModel::DIVE_IDX, i, 2, Qt::MatchRecursive);
-	QItemSelectionModel::SelectionFlags flags;
-	if (match.isEmpty())
-		return;
-	QModelIndex idx = match.first();
-	flags = toggle ? QItemSelectionModel::Toggle : QItemSelectionModel::Select;
+	QItemSelectionModel::SelectionFlags flags = toggle ? QItemSelectionModel::Toggle : QItemSelectionModel::Select;
 	flags |= QItemSelectionModel::Rows;
 	selectionModel()->setCurrentIndex(idx, flags);
 	if (idx.parent().isValid()) {
@@ -327,6 +368,18 @@ void DiveListView::selectDive(int i, bool scrollto, bool toggle)
 		scrollTo(idx, PositionAtCenter);
 }
 
+void DiveListView::selectDive(int i, bool scrollto, bool toggle)
+{
+	if (i == -1)
+		return;
+	QAbstractItemModel *m = model();
+	QModelIndexList match = m->match(m->index(0, 0), DiveTripModel::DIVE_IDX, i, 2, Qt::MatchRecursive);
+	if (match.isEmpty())
+		return;
+	QModelIndex idx = match.first();
+	selectDive(idx, scrollto, toggle);
+}
+
 void DiveListView::selectDives(const QList<int> &newDiveSelection)
 {
 	int firstInList, newSelection;
@@ -345,7 +398,7 @@ void DiveListView::selectDives(const QList<int> &newDiveSelection)
 	while (!sortedSelection.isEmpty())
 		selectDive(sortedSelection.takeLast());
 
-	while (selected_dive == -1) {
+	while (!current_dive) {
 		// that can happen if we restored a selection after edit
 		// and the only selected dive is no longer visible because of a filter
 		newSelection--;
@@ -356,8 +409,8 @@ void DiveListView::selectDives(const QList<int> &newDiveSelection)
 		if ((d = get_dive(newSelection)) != NULL && !d->hidden_by_filter)
 			selectDive(newSelection);
 	}
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
-	QModelIndexList idxList = m->match(m->index(0, 0), DiveTripModel::DIVE_IDX, selected_dive, 2, Qt::MatchRecursive);
+	QAbstractItemModel *m = model();
+	QModelIndexList idxList = m->match(m->index(0, 0), DiveTripModel::DIVE_IDX, get_divenr(current_dive), 2, Qt::MatchRecursive);
 	if (!idxList.isEmpty()) {
 		QModelIndex idx = idxList.first();
 		if (idx.parent().isValid())
@@ -365,11 +418,33 @@ void DiveListView::selectDives(const QList<int> &newDiveSelection)
 		scrollTo(idx);
 	}
 	// now that everything is up to date, update the widgets
-	emit currentDiveChanged();
+	emit diveListNotifier.selectionChanged();
 	dontEmitDiveChangedSignal = false;
 	return;
 }
 
+// Get index of first dive. This assumes that trips without dives are never shown.
+// May return an invalid index if no dive is found.
+QModelIndex DiveListView::indexOfFirstDive()
+{
+	// Fetch the first top-level item. If this is a trip, it is supposed to have at least
+	// one child. In that case return the child. Otherwise return the top-level item, which
+	// should be a dive.
+	QAbstractItemModel *m = model();
+	QModelIndex firstDiveOrTrip = m->index(0, 0);
+	if (!firstDiveOrTrip.isValid())
+		return QModelIndex();
+	QModelIndex child = m->index(0, 0, firstDiveOrTrip);
+	return child.isValid() ? child : firstDiveOrTrip;
+}
+
+void DiveListView::selectFirstDive()
+{
+	QModelIndex first = indexOfFirstDive();
+	if (first.isValid())
+		setCurrentIndex(first);
+}
+
 bool DiveListView::eventFilter(QObject *, QEvent *event)
 {
 	if (event->type() != QEvent::KeyPress)
@@ -384,65 +459,42 @@ bool DiveListView::eventFilter(QObject *, QEvent *event)
 	return true;
 }
 
-// NOTE! This loses trip selection, because while we remember the
-// dives, we don't remember the trips (see the "currentSelectedDives"
-// list). I haven't figured out how to look up the trip from the
-// index. TRIP_ROLE vs DIVE_ROLE?
-void DiveListView::headerClicked(int i)
+void DiveListView::sortIndicatorChanged(int i, Qt::SortOrder order)
 {
 	DiveTripModel::Layout newLayout = i == (int)DiveTripModel::NR ? DiveTripModel::TREE : DiveTripModel::LIST;
-	rememberSelection();
-	unselectDives();
 	/* No layout change? Just re-sort, and scroll to first selection, making sure all selections are expanded */
 	if (currentLayout == newLayout) {
-		currentOrder = (currentOrder == Qt::DescendingOrder) ? Qt::AscendingOrder : Qt::DescendingOrder;
-		sortByColumn(i, currentOrder);
+		sortByColumn(i, order);
 	} else {
 		// clear the model, repopulate with new indexes.
-		if (currentLayout == DiveTripModel::TREE) {
+		rememberSelection();
+		unselectDives();
+		if (currentLayout == DiveTripModel::TREE)
 			backupExpandedRows();
-		}
-		reload(newLayout, false);
-		currentOrder = Qt::DescendingOrder;
-		sortByColumn(i, currentOrder);
-		if (newLayout == DiveTripModel::TREE) {
+		currentLayout = newLayout;
+		MultiFilterSortModel::instance()->setLayout(newLayout);
+		sortByColumn(i, order);
+		if (newLayout == DiveTripModel::TREE)
 			restoreExpandedRows();
-		}
+		restoreSelection();
 	}
-	restoreSelection();
-	// remember the new sort column
-	sortColumn = i;
 }
 
-void DiveListView::reload(DiveTripModel::Layout layout, bool forceSort)
+void DiveListView::setSortOrder(int i, Qt::SortOrder order)
 {
-	if (layout == DiveTripModel::CURRENT)
-		layout = currentLayout;
+	// The QHeaderView will call our signal if the sort order changed
+	header()->setSortIndicator(i, order);
+}
+
+void DiveListView::reload()
+{
+	// A side-effect of setting the layout is reloading the model data
+	MultiFilterSortModel::instance()->setLayout(currentLayout);
+
+	if (amount_selected && current_dive != NULL)
+		selectDive(get_divenr(current_dive), true);
 	else
-		currentLayout = layout;
-
-	header()->setSectionsClickable(true);
-	connect(header(), SIGNAL(sectionPressed(int)), this, SLOT(headerClicked(int)), Qt::UniqueConnection);
-
-	DiveTripModel *tripModel = DiveTripModel::instance();
-	tripModel->setLayout(layout);	// Note: setLayout() resets the whole model
-
-	if (!forceSort)
-		return;
-
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
-	sortByColumn(sortColumn, currentOrder);
-	if (amount_selected && current_dive != NULL) {
-		selectDive(selected_dive, true);
-	} else {
-		QModelIndex firstDiveOrTrip = m->index(0, 0);
-		if (firstDiveOrTrip.isValid()) {
-			if (m->index(0, 0, firstDiveOrTrip).isValid())
-				setCurrentIndex(m->index(0, 0, firstDiveOrTrip));
-			else
-				setCurrentIndex(firstDiveOrTrip);
-		}
-	}
+		selectFirstDive();
 	if (selectedIndexes().count()) {
 		QModelIndex curr = selectedIndexes().first();
 		curr = curr.parent().isValid() ? curr.parent() : curr;
@@ -536,11 +588,20 @@ void DiveListView::currentChanged(const QModelIndex &current, const QModelIndex&
 
 void DiveListView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
 {
-	QItemSelection newSelected = selected.size() ? selected : selectionModel()->selection();
-	QItemSelection newDeselected = deselected;
+	if (diveListNotifier.inCommand()) {
+		// This is a programmatical change of the selection.
+		// Call the QTreeView base function to reflect the selection in the display,
+		// but don't process it any further.
+		QTreeView::selectionChanged(selected, deselected);
+		return;
+	}
 
-	disconnect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(selectionChanged(QItemSelection, QItemSelection)));
-	disconnect(selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)), this, SLOT(currentChanged(QModelIndex, QModelIndex)));
+	// This is a manual selection change. This means that the core does not yet know
+	// of the new selection. Update the core-structures accordingly and select/deselect
+	// all dives of a trip if a trip is selected/deselected.
+	const QItemSelection &newDeselected = deselected;
+
+	QItemSelection newSelected = selected.size() ? selected : selectionModel()->selection();
 
 	Q_FOREACH (const QModelIndex &index, newDeselected.indexes()) {
 		if (index.column() != 0)
@@ -550,7 +611,7 @@ void DiveListView::selectionChanged(const QItemSelection &selected, const QItemS
 		if (!dive) // it's a trip!
 			deselect_dives_in_trip((dive_trip_t *)model->data(index, DiveTripModel::TRIP_ROLE).value<void *>());
 		else
-			deselect_dive(get_divenr(dive));
+			deselect_dive(dive);
 	}
 	Q_FOREACH (const QModelIndex &index, newSelected.indexes()) {
 		if (index.column() != 0)
@@ -569,14 +630,14 @@ void DiveListView::selectionChanged(const QItemSelection &selected, const QItemS
 					expand(index);
 			}
 		} else {
-			select_dive(get_divenr(dive));
+			select_dive(dive);
 		}
 	}
-	QTreeView::selectionChanged(selectionModel()->selection(), newDeselected);
-	connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(selectionChanged(QItemSelection, QItemSelection)));
-	connect(selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)), this, SLOT(currentChanged(QModelIndex, QModelIndex)));
 	if (!dontEmitDiveChangedSignal)
-		emit currentDiveChanged();
+		emit diveListNotifier.selectionChanged();
+
+	// Display the new, processed, selection
+	QTreeView::selectionChanged(selectionModel()->selection(), newDeselected);
 }
 
 enum asked_user {NOTYET, MERGE, DONTMERGE};
@@ -611,21 +672,30 @@ static bool can_merge(const struct dive *a, const struct dive *b, enum asked_use
 void DiveListView::mergeDives()
 {
 	int i;
-	struct dive *dive, *maindive = NULL;
+	struct dive *d;
 	enum asked_user have_asked = NOTYET;
 
-	for_each_dive (i, dive) {
-		if (dive->selected) {
-			if (!can_merge(maindive, dive, &have_asked)) {
-				maindive = dive;
-			} else {
-				maindive = merge_two_dives(maindive, dive);
-				i--; // otherwise we skip a dive in the freshly changed list
-			}
+	// Collect a vector of batches of dives to merge (i.e. a vector of vector of dives)
+	QVector<QVector<dive *>> merge_batches;
+	QVector<dive *> current_batch;
+	for_each_dive (i, d) {
+		if (!d->selected)
+			continue;
+		if (current_batch.empty()) {
+			current_batch.append(d);
+		} else if (can_merge(current_batch.back(), d, &have_asked)) {
+			current_batch.append(d);
+		} else {
+			if (current_batch.count() > 1)
+				merge_batches.append(current_batch);
+			current_batch.clear();
 		}
 	}
-	MainWindow::instance()->refreshProfile();
-	MainWindow::instance()->refreshDisplay();
+	if (current_batch.count() > 1)
+		merge_batches.append(current_batch);
+
+	for (const QVector<dive *> &batch: merge_batches)
+		Command::mergeDives(batch);
 }
 
 void DiveListView::splitDives()
@@ -633,12 +703,14 @@ void DiveListView::splitDives()
 	int i;
 	struct dive *dive;
 
+	// Let's collect the dives to be split first, so that we don't catch newly inserted dives!
+	QVector<struct dive *> dives;
 	for_each_dive (i, dive) {
 		if (dive->selected)
-			split_dive(dive);
+			dives.append(dive);
 	}
-	MainWindow::instance()->refreshProfile();
-	MainWindow::instance()->refreshDisplay();
+	for (struct dive *d: dives)
+		Command::splitDives(d, duration_t{-1});
 }
 
 void DiveListView::renumberDives()
@@ -656,12 +728,7 @@ void DiveListView::merge_trip(const QModelIndex &a, int offset)
 	dive_trip_t *trip_b = (dive_trip_t *)b.data(DiveTripModel::TRIP_ROLE).value<void *>();
 	if (trip_a == trip_b || !trip_a || !trip_b)
 		return;
-	combine_trips(trip_a, trip_b);
-	rememberSelection();
-	reload(currentLayout, false);
-	restoreSelection();
-	mark_divelist_changed(true);
-	//TODO: emit a signal to signalize that the divelist changed?
+	Command::mergeTrips(trip_a, trip_b);
 }
 
 void DiveListView::mergeTripAbove()
@@ -679,21 +746,12 @@ void DiveListView::removeFromTrip()
 	//TODO: move this to C-code.
 	int i;
 	struct dive *d;
-	QMap<struct dive*, dive_trip*> divesToRemove;
+	QVector<dive *> divesToRemove;
 	for_each_dive (i, d) {
 		if (d->selected && d->divetrip)
-			divesToRemove.insert(d, d->divetrip);
+			divesToRemove.append(d);
 	}
-	if (divesToRemove.isEmpty())
-		return;
-
-	UndoRemoveDivesFromTrip *undoCommand = new UndoRemoveDivesFromTrip(divesToRemove);
-	MainWindow::instance()->undoStack->push(undoCommand);
-
-	rememberSelection();
-	reload(currentLayout, false);
-	restoreSelection();
-	mark_divelist_changed(true);
+	Command::removeDivesFromTrip(divesToRemove);
 }
 
 void DiveListView::newTripAbove()
@@ -702,18 +760,13 @@ void DiveListView::newTripAbove()
 	if (!d) // shouldn't happen as we only are setting up this action if this is a dive
 		return;
 	//TODO: port to c-code.
-	dive_trip_t *trip;
 	int idx;
-	rememberSelection();
-	trip = create_and_hookup_trip_from_dive(d);
+	QVector<dive *> dives;
 	for_each_dive (idx, d) {
 		if (d->selected)
-			add_dive_to_trip(d, trip);
+			dives.append(d);
 	}
-	trip->expanded = 1;
-	reload(currentLayout, false);
-	mark_divelist_changed(true);
-	restoreSelection();
+	Command::createTrip(dives);
 }
 
 void DiveListView::addToTripBelow()
@@ -747,21 +800,15 @@ void DiveListView::addToTrip(int delta)
 		// no dive, no trip? get me out of here
 		return;
 
-	rememberSelection();
-
-	add_dive_to_trip(d, trip);
+	QVector<dive *> dives;
 	if (d->selected) { // there are possibly other selected dives that we should add
 		int idx;
 		for_each_dive (idx, d) {
 			if (d->selected)
-				add_dive_to_trip(d, trip);
+				dives.append(d);
 		}
 	}
-	trip->expanded = 1;
-	mark_divelist_changed(true);
-
-	reload(currentLayout, false);
-	restoreSelection();
+	Command::addDivesToTrip(dives, trip);
 }
 
 void DiveListView::markDiveInvalid()
@@ -777,15 +824,10 @@ void DiveListView::markDiveInvalid()
 		// now mark the dive invalid... how do we do THAT?
 		// d->invalid = true;
 	}
-	if (amount_selected == 0) {
-		MainWindow::instance()->cleanUpEmpty();
-	}
 	mark_divelist_changed(true);
 	MainWindow::instance()->refreshDisplay();
 	if (prefs.display_invalid_dives == false) {
 		clearSelection();
-		// select top dive that isn't marked invalid
-		rememberSelection();
 	}
 }
 
@@ -796,27 +838,12 @@ void DiveListView::deleteDive()
 		return;
 
 	int i;
-	int lastDiveNr = -1;
-	QList<struct dive*> deletedDives; //a list of all deleted dives to be stored in the undo command
+	QVector<struct dive*> deletedDives;
 	for_each_dive (i, d) {
-		if (!d->selected)
-			continue;
-		deletedDives.append(d);
-		lastDiveNr = i;
-	}
-	// the actual dive deletion is happening in the redo command that is implicitly triggered
-	UndoDeleteDive *undoEntry = new UndoDeleteDive(deletedDives);
-	MainWindow::instance()->undoStack->push(undoEntry);
-	if (amount_selected == 0) {
-		MainWindow::instance()->cleanUpEmpty();
-	}
-	mark_divelist_changed(true);
-	MainWindow::instance()->refreshDisplay();
-	if (lastDiveNr != -1) {
-		clearSelection();
-		selectDive(lastDiveNr);
-		rememberSelection();
+		if (d->selected)
+			deletedDives.append(d);
 	}
+	Command::deleteDive(deletedDives);
 }
 
 void DiveListView::contextMenuEvent(QContextMenuEvent *event)
@@ -860,6 +887,7 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 			if (!d->divetrip) {
 				struct dive *top = d;
 				struct dive *bottom = d;
+				Qt::SortOrder currentOrder = header()->sortIndicatorOrder();
 				if (d->selected) {
 					if (currentOrder == Qt::AscendingOrder) {
 						top = first_selected_dive();
@@ -901,7 +929,7 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 	QAction *actionTaken = popup.exec(event->globalPos());
 	if (actionTaken == collapseAction && collapseAction) {
 		this->setAnimated(false);
-		selectDive(selected_dive, true);
+		selectDive(get_divenr(current_dive), true);
 		scrollTo(selectedIndexes().first());
 		this->setAnimated(true);
 	}
@@ -1003,6 +1031,19 @@ void DiveListView::loadImageFromURL(QUrl url)
 	}
 }
 
+void DiveListView::filterFinished()
+{
+	// first make sure the trips are no longer shown as selected
+	// (but without updating the selection state of the dives... this just cleans
+	// up an oddity in the filter handling)
+	clearTripSelection();
+
+	// If there are no more selected dives, select the first visible dive
+	if (!selectionModel()->hasSelection())
+		selectFirstDive();
+	emit diveListNotifier.selectionChanged();
+}
+
 QString DiveListView::lastUsedImageDir()
 {
 	QSettings settings;
