@@ -25,12 +25,16 @@ public:
 	void mouseDoubleClickEvent(QMouseEvent * event);
 	void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
 	void currentChanged(const QModelIndex &current, const QModelIndex &previous);
-	void reload(DiveTripModel::Layout layout, bool forceSort = true);
+	void setSortOrder(int i, Qt::SortOrder order); // Call to set sort order
+	void reload(); // Call to reload model data
 	bool eventFilter(QObject *, QEvent *);
 	void unselectDives();
 	void clearTripSelection();
+	void selectDive(QModelIndex index, bool scrollto = false, bool toggle = false);
 	void selectDive(int dive_table_idx, bool scrollto = false, bool toggle = false);
 	void selectDives(const QList<int> &newDiveSelection);
+	void selectFirstDive();
+	QModelIndex indexOfFirstDive();
 	void rememberSelection();
 	void restoreSelection();
 	void contextMenuEvent(QContextMenuEvent *event);
@@ -41,12 +45,12 @@ public
 slots:
 	void toggleColumnVisibilityByIndex();
 	void reloadHeaderActions();
-	void headerClicked(int);
+	void sortIndicatorChanged(int index, Qt::SortOrder order);
 	void removeFromTrip();
 	void deleteDive();
 	void markDiveInvalid();
-	void rowsInserted(const QModelIndex &parent, int start, int end);
-	void reset();
+	void rowsInserted(const QModelIndex &parent, int start, int end) override;
+	void reset() override;
 	void mergeTripAbove();
 	void mergeTripBelow();
 	void newTripAbove();
@@ -58,15 +62,12 @@ slots:
 	void shiftTimes();
 	void loadImages();
 	void loadWebImages();
-
-signals:
-	void currentDiveChanged();
-
+	void diveSelectionChanged(const QVector<QModelIndex> &indexes, bool select);
+	void currentDiveChanged(QModelIndex index);
+	void filterFinished();
 private:
 	bool mouseClickSelection;
 	QList<int> expandedRows;
-	int sortColumn;
-	Qt::SortOrder currentOrder;
 	DiveTripModel::Layout currentLayout;
 	QModelIndex contextMenuIndex;
 	bool dontEmitDiveChangedSignal;
