@@ -275,7 +275,7 @@ void DiveLogExportDialog::export_TeX(const char *filename, const bool selected_o
 		if (selected_only && !dive->selected)
 			continue;
 
-		ProfileWidget2 *profile = MainWindow::instance()->graphics();
+		ProfileWidget2 *profile = MainWindow::instance()->graphics;
 		profile->plotDive(dive, true);
 		profile->setToolTipVisibile(false);
 		QPixmap pix = QPixmap::grabWidget(profile);
@@ -286,7 +286,7 @@ void DiveLogExportDialog::export_TeX(const char *filename, const bool selected_o
 		struct tm tm;
 		utc_mkdate(dive->when, &tm);
 
-		dive_site *site = get_dive_site_by_uuid(dive->dive_site_uuid);
+		dive_site *site = dive->dive_site;
 		QRegExp ct("countrytag: (\\w+)");
 		QString country;
 		if (ct.indexIn(site->notes) >= 0)
@@ -315,8 +315,8 @@ void DiveLogExportDialog::export_TeX(const char *filename, const bool selected_o
 		put_format(&buf, "\\def\\place{%s}\n", site ? site->name : "");
 		put_format(&buf, "\\def\\spot{}\n");
 		put_format(&buf, "\\def\\sitename{%s}\n", site ? site->name : "");
-		site ? put_format(&buf, "\\def\\gpslat{%f}\n", site->latitude.udeg / 1000000.0) : put_format(&buf, "\\def\\gpslat{}\n");
-		site ? put_format(&buf, "\\def\\gpslon{%f}\n", site->longitude.udeg / 1000000.0) : put_format(&buf, "\\def\\gpslon{}\n");
+		site ? put_format(&buf, "\\def\\gpslat{%f}\n", site->location.lat.udeg / 1000000.0) : put_format(&buf, "\\def\\gpslat{}\n");
+		site ? put_format(&buf, "\\def\\gpslon{%f}\n", site->location.lon.udeg / 1000000.0) : put_format(&buf, "\\def\\gpslon{}\n");
 		put_format(&buf, "\\def\\computer{%s}\n", dive->dc.model);
 		put_format(&buf, "\\def\\country{%s}\n", qPrintable(country));
 		put_format(&buf, "\\def\\time{%u:%02u}\n", FRACTION(dive->duration.seconds, 60));
