@@ -67,7 +67,7 @@ class PlannerSettingsWidget : public QWidget {
 	Q_OBJECT
 public:
 	explicit PlannerSettingsWidget(QWidget *parent = 0, Qt::WindowFlags f = 0);
-	virtual ~PlannerSettingsWidget();
+	~PlannerSettingsWidget();
 public
 slots:
 	void settingsChanged();
