@@ -36,7 +36,10 @@
 #include "core/settings/qPrefPartialPressureGas.h"
 #include "core/settings/qPrefTechnicalDetails.h"
 
+#include "core/subsurface-qt/DiveListNotifier.h"
+
 #include "desktop-widgets/about.h"
+#include "desktop-widgets/command.h"
 #include "desktop-widgets/divecomputermanagementdialog.h"
 #include "desktop-widgets/divelistview.h"
 #include "desktop-widgets/divelogexportdialog.h"
@@ -66,7 +69,6 @@
 #include "qt-models/tankinfomodel.h"
 #include "qt-models/weightsysteminfomodel.h"
 #include "qt-models/yearlystatisticsmodel.h"
-
 #include "preferences/preferencesdialog.h"
 
 #ifndef NO_USERMANUAL
@@ -135,14 +137,14 @@ MainWindow::MainWindow() : QMainWindow(),
 	// widgets will change on the mainwindow.
 
 	// for the "default" mode
-	MainTab *mainTab = new MainTab();
-	DiveListView *diveListView = new DiveListView();
-	ProfileWidget2 *profileWidget = new ProfileWidget2();
+	mainTab = new MainTab(this);
+	diveList = new DiveListView(this);
+	graphics = new ProfileWidget2(this);
 	MapWidget *mapWidget = MapWidget::instance();
 
-	PlannerSettingsWidget *plannerSettings = new PlannerSettingsWidget();
-	DivePlannerWidget *plannerWidget = new DivePlannerWidget();
-	PlannerDetails *plannerDetails = new PlannerDetails();
+	divePlannerSettingsWidget = new PlannerSettingsWidget(this);
+	divePlannerWidget = new DivePlannerWidget(this);
+	plannerDetails = new PlannerDetails(this);
 
 	// what is a sane order for those icons? we should have the ones the user is
 	// most likely to want towards the top so they are always visible
@@ -168,10 +170,10 @@ MainWindow::MainWindow() : QMainWindow(),
 	profLayout->setMargin(0);
 	profLayout->setContentsMargins(0,0,0,0);
 	profLayout->addWidget(toolBar);
-	profLayout->addWidget(profileWidget);
+	profLayout->addWidget(graphics);
 	profileContainer->setLayout(profLayout);
 
-	LocationInformationWidget * diveSiteEdit = new LocationInformationWidget();
+	diveSiteEdit = new LocationInformationWidget(this);
 	connect(diveSiteEdit, &LocationInformationWidget::endEditDiveSite,
 			this, &MainWindow::setDefaultState);
 	connect(diveSiteEdit, SIGNAL(endEditDiveSite()), this, SLOT(refreshDisplay()));
@@ -185,12 +187,12 @@ MainWindow::MainWindow() : QMainWindow(),
 	enabledList.push_back(enabled);
 	disabledList.push_back(disabled);
 
-	registerApplicationState("Default", mainTab, profileContainer, diveListView, mapWidget );
-	registerApplicationState("AddDive", mainTab, profileContainer, diveListView, mapWidget );
-	registerApplicationState("EditDive", mainTab, profileContainer, diveListView, mapWidget );
-	registerApplicationState("PlanDive", plannerWidget, profileContainer, plannerSettings, plannerDetails );
-	registerApplicationState("EditPlannedDive", plannerWidget, profileContainer, diveListView, mapWidget );
-	registerApplicationState("EditDiveSite", diveSiteEdit, profileContainer, diveListView, mapWidget);
+	registerApplicationState("Default", mainTab, profileContainer, diveList, mapWidget );
+	registerApplicationState("AddDive", mainTab, profileContainer, diveList, mapWidget );
+	registerApplicationState("EditDive", mainTab, profileContainer, diveList, mapWidget );
+	registerApplicationState("PlanDive", divePlannerWidget, profileContainer, divePlannerSettingsWidget, plannerDetails );
+	registerApplicationState("EditPlannedDive", divePlannerWidget, profileContainer, diveList, mapWidget );
+	registerApplicationState("EditDiveSite", diveSiteEdit, profileContainer, diveList, mapWidget);
 
 	setStateProperties("Default", enabledList, enabledList, enabledList,enabledList);
 	setStateProperties("AddDive", enabledList, enabledList, enabledList,enabledList);
@@ -207,13 +209,13 @@ MainWindow::MainWindow() : QMainWindow(),
 	if (!QIcon::hasThemeIcon("window-close")) {
 		QIcon::setThemeName("subsurface");
 	}
-	connect(dive_list(), &DiveListView::currentDiveChanged, this, &MainWindow::current_dive_changed);
+	connect(&diveListNotifier, &DiveListNotifier::selectionChanged, this, &MainWindow::selectionChanged);
 	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), this, SLOT(readSettings()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), diveListView, SLOT(update()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), diveListView, SLOT(reloadHeaderActions()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), information(), SLOT(updateDiveInfo()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), divePlannerWidget(), SLOT(settingsChanged()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), divePlannerSettingsWidget(), SLOT(settingsChanged()));
+	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), diveList, SLOT(update()));
+	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), diveList, SLOT(reloadHeaderActions()));
+	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), mainTab, SLOT(updateDiveInfo()));
+	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), divePlannerWidget, SLOT(settingsChanged()));
+	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), divePlannerSettingsWidget, SLOT(settingsChanged()));
 	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), TankInfoModel::instance(), SLOT(update()));
 	for (int i = 0; i < NUM_RECENT_FILES; i++) {
 		actionsRecent[i] = new QAction(this);
@@ -222,14 +224,14 @@ MainWindow::MainWindow() : QMainWindow(),
 		connect(actionsRecent[i], SIGNAL(triggered(bool)), this, SLOT(recentFileTriggered(bool)));
 	}
 	ui.menuFile->insertSeparator(ui.actionQuit);
-	connect(information(), SIGNAL(addDiveFinished()), graphics(), SLOT(setProfileState()));
-	connect(information(), SIGNAL(dateTimeChanged()), graphics(), SLOT(dateTimeChanged()));
+	connect(mainTab, SIGNAL(addDiveFinished()), graphics, SLOT(setProfileState()));
+	connect(mainTab, SIGNAL(dateTimeChanged()), graphics, SLOT(dateTimeChanged()));
 	connect(DivePlannerPointsModel::instance(), SIGNAL(planCreated()), this, SLOT(planCreated()));
 	connect(DivePlannerPointsModel::instance(), SIGNAL(planCanceled()), this, SLOT(planCanceled()));
 	connect(DivePlannerPointsModel::instance(), SIGNAL(variationsComputed(QString)), this, SLOT(updateVariations(QString)));
-	connect(plannerDetails->printPlan(), SIGNAL(pressed()), divePlannerWidget(), SLOT(printDecoPlan()));
+	connect(plannerDetails->printPlan(), SIGNAL(pressed()), divePlannerWidget, SLOT(printDecoPlan()));
 	connect(this, SIGNAL(startDiveSiteEdit()), this, SLOT(on_actionDiveSiteEdit_triggered()));
-	connect(information(), SIGNAL(diveSiteChanged(struct dive_site *)), mapWidget, SLOT(centerOnDiveSite(struct dive_site *)));
+	connect(mainTab, &MainTab::diveSiteChanged, mapWidget, &MapWidget::centerOnSelectedDiveSite);
 	connect(this, &MainWindow::showError, ui.mainErrorMessage, &NotificationWidget::showError, Qt::AutoConnection);
 
 	connect(&windowTitleUpdate, &WindowTitleUpdate::updateTitle, this, &MainWindow::setAutomaticTitle);
@@ -240,17 +242,17 @@ MainWindow::MainWindow() : QMainWindow(),
 	enableDisableCloudActions();
 
 	ui.mainErrorMessage->hide();
-	graphics()->setEmptyState();
+	graphics->setEmptyState();
 	initialUiSetup();
 	readSettings();
-	diveListView->reload(DiveTripModel::TREE);
-	diveListView->reloadHeaderActions();
-	diveListView->setFocus();
+	diveList->reload();
+	diveList->reloadHeaderActions();
+	diveList->setFocus();
 	MapWidget::instance()->reload();
-	diveListView->expand(dive_list()->model()->index(0, 0));
-	diveListView->scrollTo(dive_list()->model()->index(0, 0), QAbstractItemView::PositionAtCenter);
-	divePlannerWidget()->settingsChanged();
-	divePlannerSettingsWidget()->settingsChanged();
+	diveList->expand(diveList->model()->index(0, 0));
+	diveList->scrollTo(diveList->model()->index(0, 0), QAbstractItemView::PositionAtCenter);
+	divePlannerWidget->settingsChanged();
+	divePlannerSettingsWidget->settingsChanged();
 #ifdef NO_USERMANUAL
 	ui.menuHelp->removeAction(ui.actionUserManual);
 #endif
@@ -258,19 +260,12 @@ MainWindow::MainWindow() : QMainWindow(),
 	memset(&what, 0, sizeof(what));
 
 	updateManager = new UpdateManager(this);
-	undoStack = new QUndoStack(this);
-	QAction *undoAction = undoStack->createUndoAction(this, tr("&Undo"));
-	QAction *redoAction = undoStack->createRedoAction(this, tr("&Redo"));
+	undoAction = Command::undoAction(this);
+	redoAction = Command::redoAction(this);
 	undoAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Z));
 	redoAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Z));
-	QList<QAction*>undoRedoActions;
-	undoRedoActions.append(undoAction);
-	undoRedoActions.append(redoAction);
-	ui.menu_Edit->addActions(undoRedoActions);
+	ui.menu_Edit->addActions({ undoAction, redoAction });
 
-	ReverseGeoLookupThread *geoLookup = ReverseGeoLookupThread::instance();
-	connect(geoLookup, SIGNAL(started()),information(), SLOT(disableGeoLookupEdition()));
-	connect(geoLookup, SIGNAL(finished()), information(), SLOT(enableGeoLookupEdition()));
 #ifndef NO_PRINTING
 	// copy the bundled print templates to the user path
 	QStringList templateBackupList;
@@ -322,34 +317,34 @@ MainWindow::MainWindow() : QMainWindow(),
 	connect(ui.profPn2, &QAction::triggered, pp_gas, &qPrefPartialPressureGas::set_pn2);
 	connect(ui.profPO2, &QAction::triggered, pp_gas, &qPrefPartialPressureGas::set_po2);
 
-	connect(tec, &qPrefTechnicalDetails::calcalltissuesChanged        , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::calcceilingChanged           , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::dcceilingChanged             , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::eadChanged                   , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::calcceiling3mChanged         , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::modChanged                   , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::calcndlttsChanged            , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::hrgraphChanged               , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::rulergraphChanged            , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::show_sacChanged               , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::zoomed_plotChanged            , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::show_pictures_in_profileChanged , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::tankbarChanged               , graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::percentagegraphChanged       , graphics(), &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::calcalltissuesChanged        , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::calcceilingChanged           , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::dcceilingChanged             , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::eadChanged                   , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::calcceiling3mChanged         , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::modChanged                   , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::calcndlttsChanged            , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::hrgraphChanged               , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::rulergraphChanged            , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::show_sacChanged              , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::zoomed_plotChanged           , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::show_pictures_in_profileChanged , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::tankbarChanged               , graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::percentagegraphChanged       , graphics, &ProfileWidget2::actionRequestedReplot);
 
-	connect(pp_gas, &qPrefPartialPressureGas::pheChanged, graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(pp_gas, &qPrefPartialPressureGas::pn2Changed, graphics(), &ProfileWidget2::actionRequestedReplot);
-	connect(pp_gas, &qPrefPartialPressureGas::po2Changed, graphics(), &ProfileWidget2::actionRequestedReplot);
+	connect(pp_gas, &qPrefPartialPressureGas::pheChanged, graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(pp_gas, &qPrefPartialPressureGas::pn2Changed, graphics, &ProfileWidget2::actionRequestedReplot);
+	connect(pp_gas, &qPrefPartialPressureGas::po2Changed, graphics, &ProfileWidget2::actionRequestedReplot);
 
 	// now let's set up some connections
-	connect(graphics(), &ProfileWidget2::enableToolbar ,this, &MainWindow::setEnabledToolbar);
-	connect(graphics(), &ProfileWidget2::disableShortcuts, this, &MainWindow::disableShortcuts);
-	connect(graphics(), &ProfileWidget2::enableShortcuts, this, &MainWindow::enableShortcuts);
-	connect(graphics(), &ProfileWidget2::refreshDisplay, this, &MainWindow::refreshDisplay);
-	connect(graphics(), &ProfileWidget2::editCurrentDive, this, &MainWindow::editCurrentDive);
-	connect(graphics(), &ProfileWidget2::updateDiveInfo, information(), &MainTab::updateDiveInfo);
+	connect(graphics, &ProfileWidget2::enableToolbar ,this, &MainWindow::setEnabledToolbar);
+	connect(graphics, &ProfileWidget2::disableShortcuts, this, &MainWindow::disableShortcuts);
+	connect(graphics, &ProfileWidget2::enableShortcuts, this, &MainWindow::enableShortcuts);
+	connect(graphics, &ProfileWidget2::refreshDisplay, this, &MainWindow::refreshDisplay);
+	connect(graphics, &ProfileWidget2::editCurrentDive, this, &MainWindow::editCurrentDive);
+	connect(graphics, &ProfileWidget2::updateDiveInfo, mainTab, &MainTab::updateDiveInfo);
 
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), graphics(), SLOT(settingsChanged()));
+	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), graphics, SLOT(settingsChanged()));
 
 	ui.profCalcAllTissues->setChecked(qPrefTechnicalDetails::calcalltissues());
 	ui.profCalcCeiling->setChecked(qPrefTechnicalDetails::calcceiling());
@@ -390,7 +385,7 @@ void MainWindow::setupSocialNetworkMenu()
 	connections = new QMenu(tr("Connect to"));
 	FacebookPlugin *facebookPlugin = new FacebookPlugin();
 	QAction *toggle_connection = new QAction(this);
-	QObject *obj = qobject_cast<QObject*>(facebookPlugin);
+	QObject *obj = facebookPlugin;
 	toggle_connection->setText(facebookPlugin->socialNetworkName());
 	toggle_connection->setIcon(QIcon(facebookPlugin->socialNetworkIcon()));
 	toggle_connection->setData(QVariant::fromValue(obj));
@@ -450,7 +445,9 @@ void MainWindow::setStateProperties(const QByteArray& state, const PropertyList&
 	stateProperties[state] = PropertiesForQuadrant(tl, tr, bl, br);
 }
 
-void MainWindow::on_actionDiveSiteEdit_triggered() {
+void MainWindow::on_actionDiveSiteEdit_triggered()
+{
+	diveSiteEdit->initFields(get_dive_site_for_dive(&displayed_dive));
 	setApplicationState("EditDiveSite");
 }
 
@@ -460,16 +457,16 @@ void MainWindow::enableDisableCloudActions()
 	ui.actionCloudstoragesave->setEnabled(prefs.cloud_verification_status == qPrefCloudStorage::CS_VERIFIED);
 }
 
-PlannerDetails *MainWindow::plannerDetails() const {
-	return qobject_cast<PlannerDetails*>(applicationState["PlanDive"].bottomRight);
-}
-PlannerSettingsWidget *MainWindow::divePlannerSettingsWidget() {
-	return qobject_cast<PlannerSettingsWidget*>(applicationState["PlanDive"].bottomLeft);
+void MainWindow::enableDisableOtherDCsActions()
+{
+	bool nr = number_of_computers(current_dive) > 1;
+	ui.actionNextDC->setEnabled(nr);
+	ui.actionPreviousDC->setEnabled(nr);
 }
 
 void MainWindow::setDefaultState() {
 	setApplicationState("Default");
-	if (information()->getEditMode() != MainTab::NONE) {
+	if (mainTab->getEditMode() != MainTab::NONE) {
 		ui.bottomLeft->currentWidget()->setEnabled(false);
 	}
 }
@@ -482,24 +479,22 @@ MainWindow *MainWindow::instance()
 // This gets called after one or more dives were added, edited or downloaded for a dive computer
 void MainWindow::refreshDisplay(bool doRecreateDiveList)
 {
-	information()->reload();
+	mainTab->reload();
 	TankInfoModel::instance()->update();
 	MapWidget::instance()->reload();
 	if (doRecreateDiveList)
 		recreateDiveList();
 
 	setApplicationState("Default");
-	dive_list()->setEnabled(true);
-	dive_list()->setFocus();
+	diveList->setEnabled(true);
+	diveList->setFocus();
 	WSInfoModel::instance()->updateInfo();
 	ui.actionAutoGroup->setChecked(autogroup);
-	if (amount_selected == 0)
-		cleanUpEmpty();
 }
 
 void MainWindow::recreateDiveList()
 {
-	dive_list()->reload(DiveTripModel::CURRENT);
+	diveList->reload();
 	TagFilterModel::instance()->repopulate();
 	BuddyFilterModel::instance()->repopulate();
 	LocationFilterModel::instance()->repopulate();
@@ -508,7 +503,7 @@ void MainWindow::recreateDiveList()
 }
 
 void MainWindow::configureToolbar() {
-	if (selected_dive > 0) {
+	if (current_dive) {
 		bool freeDiveMode = current_dive->dc.divemode == FREEDIVE;
 		ui.profCalcCeiling->setDisabled(freeDiveMode);
 		ui.profCalcCeiling->setDisabled(freeDiveMode);
@@ -532,12 +527,19 @@ void MainWindow::configureToolbar() {
 	}
 }
 
-void MainWindow::current_dive_changed()
+void MainWindow::selectionChanged()
 {
-	graphics()->plotDive(nullptr, false, true);
-	information()->updateDiveInfo();
-	configureToolbar();
-	MapWidget::instance()->reload();
+	if (!current_dive) {
+		mainTab->clearTabs();
+		mainTab->updateDiveInfo(true);
+		graphics->setEmptyState();
+	} else {
+		graphics->plotDive(nullptr, false, true);
+		mainTab->updateDiveInfo();
+		configureToolbar();
+		enableDisableOtherDCsActions();
+		MapWidget::instance()->reload();
+	}
 }
 
 void MainWindow::on_actionNew_triggered()
@@ -618,6 +620,8 @@ void MainWindow::on_actionCloudstorageopen_triggered()
 	if (!parse_file(fileNamePtr.data(), &dive_table))
 		setCurrentFile(fileNamePtr.data());
 	process_loaded_dives();
+	autogroup_dives();
+	Command::clear();
 	hideProgressBar();
 	refreshDisplay();
 }
@@ -642,8 +646,8 @@ void MainWindow::on_actionCloudstoragesave_triggered()
 
 	if (verbose)
 		qDebug() << "Saving cloud storage to:" << filename;
-	if (information()->isEditing())
-		information()->acceptChanges();
+	if (mainTab->isEditing())
+		mainTab->acceptChanges();
 
 	showProgressBar();
 	int error = save_dives(qPrintable(filename));
@@ -664,7 +668,7 @@ void MainWindow::on_actionCloudOnline_triggered()
 	// Refuse to go online if there is an edit in progress
 	if (!isOffline &&
 	    (DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING ||
-	    information()->isEditing())) {
+	    mainTab->isEditing())) {
 		QMessageBox::warning(this, tr("Warning"), tr("Please save or cancel the current dive edit before going online"));
 		// We didn't switch to online, therefore uncheck the checkbox
 		ui.actionCloudOnline->setChecked(false);
@@ -695,17 +699,13 @@ void MainWindow::on_actionCloudOnline_triggered()
 	updateCloudOnlineStatus();
 }
 
-ProfileWidget2 *MainWindow::graphics() const
-{
-	return qobject_cast<ProfileWidget2*>(applicationState["Default"].topRight->layout()->itemAt(1)->widget());
-}
-
 void MainWindow::cleanUpEmpty()
 {
-	information()->clearTabs();
-	information()->updateDiveInfo(true);
-	graphics()->setEmptyState();
-	dive_list()->reload(DiveTripModel::TREE);
+	mainTab->clearTabs();
+	mainTab->updateDiveInfo(true);
+	graphics->setEmptyState();
+	diveList->reload();
+	diveList->setSortOrder(DiveTripModel::NR, Qt::DescendingOrder);
 	MapWidget::instance()->reload();
 	if (!existing_filename)
 		setTitle();
@@ -715,7 +715,7 @@ void MainWindow::cleanUpEmpty()
 bool MainWindow::okToClose(QString message)
 {
 	if (DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING ||
-		information()->isEditing() ) {
+		mainTab->isEditing() ) {
 		QMessageBox::warning(this, tr("Warning"), message);
 		return false;
 	}
@@ -727,7 +727,7 @@ bool MainWindow::okToClose(QString message)
 
 void MainWindow::closeCurrentFile()
 {
-	graphics()->setEmptyState();
+	graphics->setEmptyState();
 	/* free the dives and trips */
 	clear_git_id();
 	clear_dive_file_data();
@@ -800,7 +800,7 @@ void MainWindow::enableShortcuts()
 void MainWindow::showProfile()
 {
 	enableShortcuts();
-	graphics()->setProfileState();
+	graphics->setProfileState();
 	setApplicationState("Default");
 }
 
@@ -817,9 +817,9 @@ void MainWindow::on_actionPreferences_triggered()
 
 void MainWindow::on_actionQuit_triggered()
 {
-	if (information()->isEditing()) {
-		information()->rejectChanges();
-		if (information()->isEditing())
+	if (mainTab->isEditing()) {
+		mainTab->rejectChanges();
+		if (mainTab->isEditing())
 			// didn't discard the edits
 			return;
 	}
@@ -860,7 +860,7 @@ bool MainWindow::plannerStateClean()
 		return false;
 
 	if (DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING ||
-		information()->isEditing()) {
+		mainTab->isEditing()) {
 		QMessageBox::warning(this, tr("Warning"), tr("Please save or cancel the current dive edit before trying to add a dive."));
 		return false;
 	}
@@ -871,7 +871,7 @@ void MainWindow::refreshProfile()
 {
 	showProfile();
 	configureToolbar();
-	graphics()->replot(get_dive(selected_dive));
+	graphics->replot(current_dive);
 	DivePictureModel::instance()->updateDivePictures();
 }
 
@@ -885,24 +885,16 @@ void MainWindow::planCanceled()
 
 void MainWindow::planCreated()
 {
-	// get the new dive selected and assign a number if reasonable
-	graphics()->setProfileState();
-	if (displayed_dive.id == 0) {
-		// we might have added a new dive (so displayed_dive was cleared out by clone_dive()
-		dive_list()->unselectDives();
-		select_dive(dive_table.nr - 1);
-		dive_list()->selectDive(selected_dive);
-		set_dive_nr_for_current_dive();
-	}
 	// make sure our UI is in a consistent state
-	information()->updateDiveInfo();
 	showProfile();
-	refreshDisplay();
+	setApplicationState("Default");
+	diveList->setEnabled(true);
+	diveList->setFocus();
 }
 
 void MainWindow::setPlanNotes()
 {
-	plannerDetails()->divePlanOutput()->setHtml(displayed_dive.notes);
+	plannerDetails->divePlanOutput()->setHtml(displayed_dive.notes);
 }
 
 void MainWindow::updateVariations(QString variations)
@@ -910,13 +902,13 @@ void MainWindow::updateVariations(QString variations)
 	QString notes = QString(displayed_dive.notes);
 	free(displayed_dive.notes);
 	displayed_dive.notes = copy_qstring(notes.replace("VARIATIONS", variations));
-	plannerDetails()->divePlanOutput()->setHtml(displayed_dive.notes);
+	plannerDetails->divePlanOutput()->setHtml(displayed_dive.notes);
 }
 
 void MainWindow::printPlan()
 {
 #ifndef NO_PRINTING
-	QString diveplan = plannerDetails()->divePlanOutput()->toHtml();
+	QString diveplan = plannerDetails->divePlanOutput()->toHtml();
 	QString withDisclaimer = QString("<img height=50 src=\":subsurface-icon\"> ") + diveplan + QString(disclaimer);
 
 	QPrinter printer;
@@ -939,7 +931,7 @@ void MainWindow::printPlan()
 	painter.setRenderHint(QPainter::Antialiasing);
 	painter.setRenderHint(QPainter::SmoothPixmapTransform);
 
-	ProfileWidget2 *profile = graphics();
+	ProfileWidget2 *profile = graphics;
 	QSize origSize = profile->size();
 	profile->resize(renderSize.toSize());
 	profile->setPrintMode(true);
@@ -953,9 +945,9 @@ void MainWindow::printPlan()
 	QString profileImage = QString("<img src=\"data:image/png;base64,") + byteArray.toBase64() + "\"/><br><br>";
 	withDisclaimer = profileImage + withDisclaimer;
 
-	plannerDetails()->divePlanOutput()->setHtml(withDisclaimer);
-	plannerDetails()->divePlanOutput()->print(&printer);
-	plannerDetails()->divePlanOutput()->setHtml(displayed_dive.notes);
+	plannerDetails->divePlanOutput()->setHtml(withDisclaimer);
+	plannerDetails->divePlanOutput()->print(&printer);
+	plannerDetails->divePlanOutput()->setHtml(displayed_dive.notes);
 #endif
 }
 
@@ -963,7 +955,6 @@ void MainWindow::setupForAddAndPlan(const char *model)
 {
 	// clean out the dive and give it an id and the correct dc model
 	clear_dive(&displayed_dive);
-	clear_dive_site(&displayed_dive_site);
 	displayed_dive.id = dive_getUniqID();
 	displayed_dive.when = QDateTime::currentMSecsSinceEpoch() / 1000L + gettimezoneoffset() + 3600;
 	displayed_dive.dc.model = strdup(model); // don't translate! this is stored in the XML file
@@ -987,15 +978,15 @@ void MainWindow::on_actionReplanDive_triggered()
 	DivePlannerPointsModel::instance()->clear();
 	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
 
-	graphics()->setPlanState();
-	graphics()->clearHandlers();
+	graphics->setPlanState();
+	graphics->clearHandlers();
 	setApplicationState("PlanDive");
-	divePlannerWidget()->setReplanButton(true);
-	divePlannerWidget()->setupStartTime(QDateTime::fromMSecsSinceEpoch(1000 * current_dive->when, Qt::UTC));
+	divePlannerWidget->setReplanButton(true);
+	divePlannerWidget->setupStartTime(QDateTime::fromMSecsSinceEpoch(1000 * current_dive->when, Qt::UTC));
 	if (current_dive->surface_pressure.mbar)
-		divePlannerWidget()->setSurfacePressure(current_dive->surface_pressure.mbar);
+		divePlannerWidget->setSurfacePressure(current_dive->surface_pressure.mbar);
 	if (current_dive->salinity)
-		divePlannerWidget()->setSalinity(current_dive->salinity);
+		divePlannerWidget->setSalinity(current_dive->salinity);
 	DivePlannerPointsModel::instance()->loadFromDive(current_dive);
 	reset_cylinders(&displayed_dive, true);
 	CylindersModel::instance()->updateDive();
@@ -1010,7 +1001,7 @@ void MainWindow::on_actionDivePlanner_triggered()
 	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
 	setApplicationState("PlanDive");
 
-	graphics()->setPlanState();
+	graphics->setPlanState();
 
 	// create a simple starting dive, using the first gas from the just copied cylinders
 	setupForAddAndPlan("planned dive"); // don't translate, stored in XML file
@@ -1018,16 +1009,12 @@ void MainWindow::on_actionDivePlanner_triggered()
 	DivePlannerPointsModel::instance()->createSimpleDive();
 	// plan the dive in the same mode as the currently selected one
 	if (current_dive) {
-		divePlannerSettingsWidget()->setDiveMode(current_dive->dc.divemode);
+		divePlannerSettingsWidget->setDiveMode(current_dive->dc.divemode);
 		if (current_dive->salinity)
-			divePlannerWidget()->setSalinity(current_dive->salinity);
+			divePlannerWidget->setSalinity(current_dive->salinity);
 	}
 	DivePictureModel::instance()->updateDivePictures();
-	divePlannerWidget()->setReplanButton(false);
-}
-
-DivePlannerWidget* MainWindow::divePlannerWidget() {
-	return qobject_cast<DivePlannerWidget*>(applicationState["PlanDive"].topLeft);
+	divePlannerWidget->setReplanButton(false);
 }
 
 void MainWindow::on_actionAddDive_triggered()
@@ -1035,9 +1022,9 @@ void MainWindow::on_actionAddDive_triggered()
 	if (!plannerStateClean())
 		return;
 
-	if (dive_list()->selectedTrips().count() >= 1) {
-		dive_list()->rememberSelection();
-		dive_list()->clearSelection();
+	if (diveList->selectedTrips().count() >= 1) {
+		diveList->rememberSelection();
+		diveList->clearSelection();
 	}
 
 	setApplicationState("AddDive");
@@ -1047,19 +1034,19 @@ void MainWindow::on_actionAddDive_triggered()
 	setupForAddAndPlan("manually added dive"); // don't translate, stored in the XML file
 
 	// now show the mostly empty main tab
-	information()->updateDiveInfo();
+	mainTab->updateDiveInfo();
 
-	information()->addDiveStarted();
+	mainTab->addDiveStarted();
 
-	graphics()->setAddState();
+	graphics->setAddState();
 	DivePlannerPointsModel::instance()->createSimpleDive();
 	configureToolbar();
-	graphics()->plotDive(nullptr, false, true);
+	graphics->plotDive(nullptr, false, true);
 	fixup_dc_duration(&displayed_dive.dc);
 	displayed_dive.duration = displayed_dive.dc.duration;
 
 	// now that we have the correct depth and duration, update the dive info
-	information()->updateDepthDuration();
+	mainTab->updateDepthDuration();
 }
 
 void MainWindow::on_actionRenumber_triggered()
@@ -1072,9 +1059,9 @@ void MainWindow::on_actionAutoGroup_triggered()
 {
 	set_autogroup(ui.actionAutoGroup->isChecked());
 	if (autogroup)
-		autogroup_dives();
+		Command::autogroupDives();
 	else
-		remove_autogen_trips();
+		Command::removeAutogenTrips();
 	refreshDisplay();
 	mark_divelist_changed(true);
 }
@@ -1182,6 +1169,8 @@ void MainWindow::on_actionViewAll_triggered()
 
 void MainWindow::enterEditState()
 {
+	undoAction->setEnabled(false);
+	redoAction->setEnabled(false);
 	stateBeforeEdit = state;
 	if (state == VIEWALL || state == INFO_MAXIMIZED)
 		return;
@@ -1205,6 +1194,8 @@ void MainWindow::enterEditState()
 
 void MainWindow::exitEditState()
 {
+	undoAction->setEnabled(true);
+	redoAction->setEnabled(true);
 	if (stateBeforeEdit == state)
 		return;
 	enterState(stateBeforeEdit);
@@ -1256,8 +1247,8 @@ void MainWindow::on_actionPreviousDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + nrdc - 1) % nrdc;
 	configureToolbar();
-	graphics()->plotDive(nullptr, false, true);
-	information()->updateDiveInfo();
+	graphics->plotDive(nullptr, false, true);
+	mainTab->updateDiveInfo();
 }
 
 void MainWindow::on_actionNextDC_triggered()
@@ -1265,8 +1256,8 @@ void MainWindow::on_actionNextDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + 1) % nrdc;
 	configureToolbar();
-	graphics()->plotDive(nullptr, false, true);
-	information()->updateDiveInfo();
+	graphics->plotDive(nullptr, false, true);
+	mainTab->updateDiveInfo();
 }
 
 void MainWindow::on_actionFullScreen_triggered(bool checked)
@@ -1497,7 +1488,7 @@ void MainWindow::writeSettings()
 void MainWindow::closeEvent(QCloseEvent *event)
 {
 	if (DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING ||
-		information()->isEditing()) {
+		mainTab->isEditing()) {
 		on_actionQuit_triggered();
 		event->ignore();
 		return;
@@ -1512,16 +1503,6 @@ void MainWindow::closeEvent(QCloseEvent *event)
 	QApplication::closeAllWindows();
 }
 
-DiveListView *MainWindow::dive_list()
-{
-	return qobject_cast<DiveListView*>(applicationState["Default"].bottomLeft);
-}
-
-MainTab *MainWindow::information()
-{
-	return qobject_cast<MainTab*>(applicationState["Default"].topLeft);
-}
-
 void MainWindow::loadRecentFiles()
 {
 	recentFiles.clear();
@@ -1640,8 +1621,8 @@ int MainWindow::file_save_as(void)
 	if (filename.isNull() || filename.isEmpty())
 		return report_error("No filename to save into");
 
-	if (information()->isEditing())
-		information()->acceptChanges();
+	if (mainTab->isEditing())
+		mainTab->acceptChanges();
 
 	if (save_dives(qPrintable(filename)))
 		return -1;
@@ -1664,8 +1645,8 @@ int MainWindow::file_save(void)
 	if (is_cloud && !saveToCloudOK())
 		return -1;
 
-	if (information()->isEditing())
-		information()->acceptChanges();
+	if (mainTab->isEditing())
+		mainTab->acceptChanges();
 
 	current_default = prefs.default_filename;
 	if (strcmp(existing_filename, current_default) == 0) {
@@ -1733,42 +1714,15 @@ void MainWindow::importFiles(const QStringList fileNames)
 		return;
 
 	QByteArray fileNamePtr;
+	struct dive_table table = { 0 };
 
 	for (int i = 0; i < fileNames.size(); ++i) {
 		fileNamePtr = QFile::encodeName(fileNames.at(i));
-		parse_file(fileNamePtr.data(), &dive_table);
+		parse_file(fileNamePtr.data(), &table);
 	}
-	process_imported_dives(false);
-	refreshDisplay();
-}
-
-void MainWindow::importTxtFiles(const QStringList fileNames)
-{
-	QStringList csvFiles;
-
-	if (fileNames.isEmpty())
-		return;
-
-	QByteArray fileNamePtr, csv;
-
-	for (int i = 0; i < fileNames.size(); ++i) {
-		fileNamePtr = QFile::encodeName(fileNames.at(i));
-		csv = fileNamePtr.data();
-		csv.replace(csv.size() - 3, 3, "csv");
-
-		QFileInfo check_file(csv);
-		if (check_file.exists() && check_file.isFile()) {
-			if (parse_txt_file(fileNamePtr.data(), csv, &dive_table) == 0)
-				csvFiles += fileNames.at(i);
-		} else {
-			csvFiles += fileNamePtr;
-		}
-	}
-	if (csvFiles.size()) {
-		DiveLogImportDialog *diveLogImport = new DiveLogImportDialog(csvFiles, this);
-		diveLogImport->show();
-	}
-	process_imported_dives(false);
+	process_imported_dives(&table, false, false);
+	autogroup_dives();
+	Command::clear();
 	refreshDisplay();
 }
 
@@ -1791,6 +1745,8 @@ void MainWindow::loadFiles(const QStringList fileNames)
 	hideProgressBar();
 	updateRecentFiles();
 	process_loaded_dives();
+	autogroup_dives();
+	Command::clear();
 
 	refreshDisplay();
 
@@ -1806,6 +1762,19 @@ void MainWindow::loadFiles(const QStringList fileNames)
 	}
 }
 
+static const char *csvExtensions[] = {
+	".csv", ".apd", ".zxu", ".zxl", ".txt"
+};
+
+static bool isCsvFile(const QString &s)
+{
+	for (const char *ext: csvExtensions) {
+		if (s.endsWith(ext, Qt::CaseInsensitive))
+			return true;
+	}
+	return false;
+}
+
 void MainWindow::on_actionImportDiveLog_triggered()
 {
 	QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Open dive log file"), lastUsedDir(), filter_import());
@@ -1814,26 +1783,22 @@ void MainWindow::on_actionImportDiveLog_triggered()
 		return;
 	updateLastUsedDir(QFileInfo(fileNames[0]).dir().path());
 
-	QStringList logFiles = fileNames.filter(QRegExp("^(?!.*\\.(csv|txt|apd|zxu|zxl))", Qt::CaseInsensitive));
-	QStringList csvFiles = fileNames.filter(".csv", Qt::CaseInsensitive);
-	csvFiles += fileNames.filter(".apd", Qt::CaseInsensitive);
-	csvFiles += fileNames.filter(".zxu", Qt::CaseInsensitive);
-	csvFiles += fileNames.filter(".zxl", Qt::CaseInsensitive);
-	QStringList txtFiles = fileNames.filter(".txt", Qt::CaseInsensitive);
+	QStringList logFiles;
+	QStringList csvFiles;
+	for (const QString &fn: fileNames) {
+		if (isCsvFile(fn))
+			csvFiles.append(fn);
+		else
+			logFiles.append(fn);
+	}
 
 	if (logFiles.size()) {
 		importFiles(logFiles);
 	}
 
 	if (csvFiles.size()) {
-		DiveLogImportDialog *diveLogImport = new DiveLogImportDialog(csvFiles, this);
-		diveLogImport->show();
-		process_imported_dives(false);
-		refreshDisplay();
-	}
-
-	if (txtFiles.size()) {
-		importTxtFiles(txtFiles);
+		DiveLogImportDialog diveLogImport(csvFiles, this);
+		diveLogImport.exec();
 	}
 }
 
@@ -1842,7 +1807,7 @@ void MainWindow::editCurrentDive()
 	if (!current_dive)
 		return;
 
-	if (information()->isEditing() || DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING) {
+	if (mainTab->isEditing() || DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING) {
 		QMessageBox::warning(this, tr("Warning"), tr("Please, first finish the current edition before trying to do another."));
 		return;
 	}
@@ -1853,18 +1818,18 @@ void MainWindow::editCurrentDive()
 	disableShortcuts();
 	if (defaultDC == "manually added dive") {
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
-		graphics()->setAddState();
+		graphics->setAddState();
 		setApplicationState("EditDive");
 		DivePlannerPointsModel::instance()->loadFromDive(d);
-		information()->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
+		mainTab->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
 	} else if (defaultDC == "planned dive") {
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
 		setApplicationState("EditPlannedDive");
 		DivePlannerPointsModel::instance()->loadFromDive(d);
-		information()->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
+		mainTab->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
 	} else {
 		setApplicationState("EditDive");
-		information()->enableEdition();
+		mainTab->enableEdition();
 	}
 }
 
@@ -1906,7 +1871,7 @@ void MainWindow::on_paste_triggered()
 {
 	// take the data in our copyPasteDive and apply it to selected dives
 	selective_copy_dive(&copyPasteDive, &displayed_dive, what, false);
-	information()->showAndTriggerEditSelective(what);
+	mainTab->showAndTriggerEditSelective(what);
 }
 
 void MainWindow::on_actionFilterTags_triggered()
