@@ -191,9 +191,8 @@ void PrintDialog::printClicked(void)
 	}
 }
 
-void PrintDialog::onPaintRequested(QPrinter *printerPtr)
+void PrintDialog::onPaintRequested(QPrinter*)
 {
-	Q_UNUSED(printerPtr)
 	createPrinterObj();
 	connect(printer, SIGNAL(progessUpdated(int)), progressBar, SLOT(setValue(int)));
 	printer->print();
