@@ -18,7 +18,7 @@
 #include "desktop-widgets/divelistview.h"
 #include "core/display.h"
 #include "profile-widget/profilewidget2.h"
-#include "desktop-widgets/undocommands.h"
+#include "desktop-widgets/command.h"
 #include "core/metadata.h"
 
 class MinMaxAvgWidgetPrivate {
@@ -159,23 +159,19 @@ void RenumberDialog::renumberOnlySelected(bool selected)
 void RenumberDialog::buttonClicked(QAbstractButton *button)
 {
 	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole) {
-		MainWindow::instance()->dive_list()->rememberSelection();
-		// we remember a map from dive uuid to a pair of old number / new number
-		QMap<int, QPair<int, int>> renumberedDives;
+		MainWindow::instance()->diveList->rememberSelection();
+		// we remember a list from dive uuid to a new number
+		QVector<QPair<dive *, int>> renumberedDives;
 		int i;
 		int newNr = ui.spinBox->value();
-		struct dive *dive = NULL;
-		for_each_dive (i, dive) {
-			if (!selectedOnly || dive->selected) {
-				invalidate_dive_cache(dive);
-				renumberedDives.insert(dive->id, QPair<int, int>(dive->number, newNr++));
+		struct dive *d;
+		for_each_dive (i, d) {
+			if (!selectedOnly || d->selected) {
+				invalidate_dive_cache(d);
+				renumberedDives.append({ d, newNr++ });
 			}
 		}
-		UndoRenumberDives *undoCommand = new UndoRenumberDives(renumberedDives);
-		MainWindow::instance()->undoStack->push(undoCommand);
-
-		mark_divelist_changed(true);
-		MainWindow::instance()->dive_list()->restoreSelection();
+		Command::renumberDives(renumberedDives);
 	}
 }
 
@@ -209,7 +205,7 @@ void SetpointDialog::buttonClicked(QAbstractButton *button)
 		invalidate_dive_cache(current_dive);
 	}
 	mark_divelist_changed(true);
-	MainWindow::instance()->graphics()->replot();
+	MainWindow::instance()->graphics->replot();
 }
 
 SetpointDialog::SetpointDialog(QWidget *parent) : QDialog(parent),
@@ -240,20 +236,13 @@ void ShiftTimesDialog::buttonClicked(QAbstractButton *button)
 		if (amount != 0) {
 			// DANGER, DANGER - this could get our dive_table unsorted...
 			int i;
-			struct dive *dive;
-			QList<int> affectedDives;
-			for_each_dive (i, dive) {
-				if (!dive->selected)
-					continue;
-
-				affectedDives.append(dive->id);
+			struct dive *d;
+			QVector<dive *> affectedDives;
+			for_each_dive (i, d) {
+				if (d->selected)
+					affectedDives.append(d);
 			}
-			MainWindow::instance()->undoStack->push(new UndoShiftTime(affectedDives, amount));
-			sort_table(&dive_table);
-			mark_divelist_changed(true);
-			MainWindow::instance()->dive_list()->rememberSelection();
-			MainWindow::instance()->refreshDisplay();
-			MainWindow::instance()->dive_list()->restoreSelection();
+			Command::shiftTime(affectedDives, amount);
 		}
 	}
 }
