@@ -876,11 +876,9 @@ bool QMLManager::checkLocation(DiveObjectHelper *myDive, struct dive *d, QString
 	qDebug() << "checkLocation" << location << "gps" << gps << "dive had" << myDive->location() << "gps" << myDive->gas();
 	if (myDive->location() != location) {
 		diveChanged = true;
-		if (!ds)
-			ds = get_dive_site_by_name(qPrintable(location));
-		if (!ds && !location.isEmpty()) {
+		ds = get_dive_site_by_name(qPrintable(location));
+		if (!ds && !location.isEmpty())
 			ds = create_dive_site(qPrintable(location), d->when);
-		}
 		d->dive_site = ds;
 	}
 	// now make sure that the GPS coordinates match - if the user changed the name but not
@@ -1637,6 +1635,17 @@ void QMLManager::setStatusbarColor(QColor)
 
 #endif
 
+void QMLManager::retrieveBluetoothName()
+{
+	QString name = DC_devName();
+	QList<BTDiscovery::btVendorProduct> btDCs = BTDiscovery::instance()->getBtDcs();
+	foreach (BTDiscovery::btVendorProduct btDC, btDCs) {
+		qDebug() << "compare" <<name << btDC.btpdi.address;
+		if (name.contains(btDC.btpdi.address))
+			DC_setDevBluetoothName(btDC.btpdi.name);
+	}
+}
+
 QString QMLManager::DC_vendor() const
 {
 	return m_device_data->vendor();
