@@ -17,26 +17,33 @@ exec 1> >(tee ./build.log) 2>&1
 
 USE_X=$(case $- in *x*) echo "-x" ;; esac)
 
-# these are the current versions for Qt, Android SDK & NDK:
-source subsurface/packaging/android/variables.sh
+# deal with the command line arguments
+while [[ $# -gt 0 ]] ; do
+        arg="$1"
+        case $arg in
+                -prep-only)
+			# only download the dependencies, don't build
+			PREP_ONLY="1"
+			;;
+		release|Release)
+			# simply pass through to build.sh
+			;;
+		*)
+			echo "Unknown command line argument $arg"
+			echo "Usage: $0 [-prep-only]"
+			exit 1
+			;;
+	esac
+	shift
+done
 
-# avoid timeouts on Travis when downloads take a long time
-SLOW_PROG=""
-if [ -n "${TRAVIS:-}" ]; then
-	source subsurface/scripts/travis-wait.sh
-	set -x # make debugging Travis easier
-	SLOW_PROG="travis_wait"
-	# since we are running on Travis, let's just get our minimal Qt install
-	mkdir -p Qt/"${LATEST_QT}"
-	$SLOW_PROG wget -q https://storage.googleapis.com/travis-cache/Qt-"${LATEST_QT}"-android.tar.xz
-	tar -xJ -C Qt/"${LATEST_QT}" -f Qt-"${LATEST_QT}"-android.tar.xz
-fi
+# these are the current versions for Qt, Android SDK & NDK:
+SCRIPTDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" > /dev/null && pwd )"
+source "$SCRIPTDIR"/variables.sh
 
 PLATFORM=$(uname)
 
-pushd $(dirname "$0")/../../
-export SUBSURFACE_SOURCE=$PWD
-popd
+export SUBSURFACE_SOURCE="$SCRIPTDIR"/../..
 
 if [ "$PLATFORM" = Linux ] ; then
 	QT_BINARIES=qt-opensource-linux-x64-${LATEST_QT}.run
@@ -68,25 +75,28 @@ fi
 # first we need to get the Android SDK and NDK
 if [ ! -d "$ANDROID_NDK" ] ; then
 	if [ ! -f "$NDK_BINARIES" ] ; then
-		$SLOW_PROG wget -q https://dl.google.com/android/repository/"$NDK_BINARIES"
+		wget -q https://dl.google.com/android/repository/"$NDK_BINARIES"
 	fi
 	unzip -q "$NDK_BINARIES"
 fi
 
-if [ ! -d "$ANDROID_SDK"/build-tools/"${ANDROID_BUILDTOOLS_REVISION}" ] ; then
+if [ ! -d "$ANDROID_SDK"/build-tools/"${ANDROID_BUILDTOOLS_REVISION}" ] ||
+   [ ! -d "$ANDROID_SDK"/platforms/android-"${ANDROID_PLATFORMS}" ] ||
+   [ ! -d "$ANDROID_SDK"/platforms/android-"${ANDROID_PLATFORM}" ] ; then
 	if [ ! -d "$ANDROID_SDK" ] ; then
 		if [ ! -f "$SDK_TOOLS" ] ; then
-			$SLOW_PROG wget -q https://dl.google.com/android/repository/"$SDK_TOOLS"
+			wget -q https://dl.google.com/android/repository/"$SDK_TOOLS"
 		fi
 		mkdir "$ANDROID_SDK"
 		pushd "$ANDROID_SDK"
 		unzip -q ../"$SDK_TOOLS"
 		yes | tools/bin/sdkmanager --licenses > /dev/null 2>&1 || echo "d56f5187479451eabf01fb78af6dfcb131a6481e" > licenses/android-sdk-license
 		cat licenses/android-sdk-license
+		yes | tools/bin/sdkmanager tools platform-tools 'platforms;'"${ANDROID_PLATFORM}" 'platforms;'"${ANDROID_PLATFORMS}" 'build-tools;'"${ANDROID_BUILDTOOLS_REVISION}" > /dev/null
 		echo ""
 	else
 		pushd "$ANDROID_SDK"
-		tools/bin/sdkmanager tools platform-tools 'platforms;'"${ANDROID_PLATFORMS}" 'build-tools;'"${ANDROID_BUILDTOOLS_REVISION}"
+		yes | tools/bin/sdkmanager tools platform-tools 'platforms;'"${ANDROID_PLATFORM}" 'platforms;'"${ANDROID_PLATFORMS}" 'build-tools;'"${ANDROID_BUILDTOOLS_REVISION}" > /dev/null
 	fi
 	popd
 fi
@@ -101,15 +111,19 @@ if [ ! -d Qt/"${LATEST_QT}"/android_armv7 ] ; then
 		echo "Qt installation found, backing it up to Qt_OLD."
 	fi
 	if [ ! -f "${QT_BINARIES}" ] ; then
-		$SLOW_PROG wget -q "${QT_DOWNLOAD_URL}"
+		wget -q "${QT_DOWNLOAD_URL}"
 	fi
 	chmod +x ./"${QT_BINARIES}"
-	./"${QT_BINARIES}" --platform minimal --script "$SUBSURFACE_SOURCE"/qt-installer-noninteractive.qs --no-force-installations
+	./"${QT_BINARIES}" --platform minimal --script "$SCRIPTDIR"/qt-installer-noninteractive.qs --no-force-installations
 fi
 
 # patch the cmake / Qt5.7.1 incompatibility mentioned above
 sed -i 's/set_property(TARGET Qt5::Core PROPERTY INTERFACE_COMPILE_FEATURES cxx_decltype)/# set_property(TARGET Qt5::Core PROPERTY INTERFACE_COMPILE_FEATURES cxx_decltype)/' Qt/"${LATEST_QT}"/android_armv7/lib/cmake/Qt5Core/Qt5CoreConfigExtras.cmake
 
+if [ ! -z ${PREP_ONLY+x} ] ; then
+	exit 0
+fi
+
 if [ ! -d subsurface/libdivecomputer/src ] ; then
 	pushd subsurface
 	git submodule init
@@ -117,12 +131,12 @@ if [ ! -d subsurface/libdivecomputer/src ] ; then
 	popd
 fi
 
-if [ ! -f subsurface/libdivecomputer/configure ] ; then
-	pushd subsurface/libdivecomputer
-	autoreconf --install
-	autoreconf --install
-	popd
-fi
+# always reconfigure here
+rm -f subsurface/libdivecomputer/configure
+pushd subsurface/libdivecomputer
+autoreconf --install --force
+autoreconf --install --force
+popd
 
 # and now we need a monotonic build number...
 if [ ! -f ./buildnr.dat ] ; then
@@ -140,8 +154,10 @@ rm -df ./subsurface-mobile-build-arm/AndroidManifest.xml
 
 if [ "$USE_X" ] ; then
 	bash "$USE_X" "$SUBSURFACE_SOURCE"/packaging/android/build.sh -buildnr "$BUILDNR" arm "$@"
+	bash "$USE_X" "$SUBSURFACE_SOURCE"/packaging/android/build.sh -buildnr "$BUILDNR" arm64 "$@"
 else
 	bash "$SUBSURFACE_SOURCE"/packaging/android/build.sh -buildnr "$BUILDNR" arm "$@"
+	bash "$SUBSURFACE_SOURCE"/packaging/android/build.sh -buildnr "$BUILDNR" arm64 "$@"
 fi
 
 ls -l ./subsurface-mobile-build-arm/build/outputs/apk/*.apk
