@@ -1,11 +1,13 @@
 #!/bin/bash
 # When changing Qt version remember to update the 
 # qt-installer-noninteractive file as well.
-QT_VERSION=5.11
-LATEST_QT=5.11.1
-NDK_VERSION=r14b
-SDK_VERSION=3859397
-ANDROID_BUILDTOOLS_REVISION=25.0.3
+QT_VERSION=5.12
+LATEST_QT=5.12.0
+NDK_VERSION=r18b
+SDK_VERSION=4333796
+ANDROID_BUILDTOOLS_REVISION=28.0.2
+ANDROID_PLATFORM_LEVEL=21
+ANDROID_PLATFORM=android-21
 ANDROID_PLATFORMS=android-27
 ANDROID_NDK=android-ndk-${NDK_VERSION}
 ANDROID_SDK=android-sdk-linux
