@@ -23,6 +23,7 @@
 #include "desktop-widgets/diveplanner.h"
 #include "desktop-widgets/simplewidgets.h"
 #include "desktop-widgets/divepicturewidget.h"
+#include "desktop-widgets/command.h"
 #include "desktop-widgets/mainwindow.h"
 #include "core/qthelper.h"
 #include "core/gettextfromc.h"
@@ -558,7 +559,7 @@ void ProfileWidget2::plotDive(struct dive *d, bool force, bool doClearPictures)
 #endif
 	if (currentState != ADD && currentState != PLAN) {
 		if (!d) {
-			if (selected_dive == -1)
+			if (!current_dive)
 				return;
 			d = current_dive; // display the current dive
 		}
@@ -734,7 +735,16 @@ void ProfileWidget2::plotDive(struct dive *d, bool force, bool doClearPictures)
 		ccrsensor2GasItem->setVisible(prefs.show_ccr_sensors && (currentdc->no_o2sensors > 1));
 		ccrsensor3GasItem->setVisible(prefs.show_ccr_sensors && (currentdc->no_o2sensors > 1));
 		ocpo2GasItem->setVisible((currentdc->divemode == PSCR) && prefs.show_scr_ocpo2);
-		temperatureItem->setVisible(false);
+		//when no gas graph, we can show temperature
+		if (!po2GasItem->isVisible() &&
+		    !o2SetpointGasItem->isVisible() &&
+		    !ccrsensor1GasItem->isVisible() &&
+		    !ccrsensor2GasItem->isVisible() &&
+		    !ccrsensor3GasItem->isVisible() &&
+		    !ocpo2GasItem->isVisible())
+			temperatureItem->setVisible(true);
+		else
+			temperatureItem->setVisible(false);
 	} else {
 		tankItem->setVisible(prefs.tankbar);
 		gasYAxis->setPos(itemPos.partialPressure.pos.off);
@@ -818,6 +828,8 @@ void ProfileWidget2::plotDive(struct dive *d, bool force, bool doClearPictures)
 		clearPictures();
 	else
 		plotPictures();
+
+	toolTipItem->refresh(mapToScene(mapFromGlobal(QCursor::pos())));
 #endif
 
 	// OK, how long did this take us? Anything above the second is way too long,
@@ -1056,7 +1068,8 @@ void ProfileWidget2::scrollViewTo(const QPoint &pos)
 void ProfileWidget2::mouseMoveEvent(QMouseEvent *event)
 {
 	QPointF pos = mapToScene(event->pos());
-	toolTipItem->refresh(pos);
+	toolTipItem->refresh(mapToScene(mapFromGlobal(QCursor::pos())));
+
 	if (zoomLevel == 0) {
 		QGraphicsView::mouseMoveEvent(event);
 	} else {
@@ -1403,7 +1416,7 @@ void ProfileWidget2::contextMenuEvent(QContextMenuEvent *event)
 	}
 	QMenu m;
 	bool isDCName = false;
-	if (selected_dive == -1)
+	if (!current_dive)
 		return;
 	// figure out if we are ontop of the dive computer name in the profile
 	QGraphicsItem *sceneItem = itemAt(mapFromGlobal(event->globalPos()));
@@ -1666,16 +1679,18 @@ void ProfileWidget2::addSetpointChange()
 
 void ProfileWidget2::splitDive()
 {
+#ifndef SUBSURFACE_MOBILE
+	// Make sure that this is an actual dive and we're not in add mode
+	dive *d = get_dive_by_uniq_id(displayed_dive.id);
+	if (!d)
+		return;
 	QAction *action = qobject_cast<QAction *>(sender());
 	QPointF scenePos = mapToScene(mapFromGlobal(action->data().toPoint()));
 	duration_t time;
-	time.seconds = lrint(timeAxis->valueAt((scenePos)));
-	split_dive_at_time(&displayed_dive, time);
+	time.seconds = lrint(timeAxis->valueAt(scenePos));
+	Command::splitDives(d, time);
 	emit updateDiveInfo(false);
-	mark_divelist_changed(true);
-	replot();
-	MainWindow::instance()->refreshProfile();
-	MainWindow::instance()->refreshDisplay();
+#endif
 }
 
 void ProfileWidget2::changeGas()
