@@ -22,10 +22,10 @@ public:
 signals:
 
 public slots:
-	virtual void modelDataChanged(const QModelIndex &topLeft = QModelIndex(), const QModelIndex &bottomRight = QModelIndex());
+	void modelDataChanged(const QModelIndex &topLeft = QModelIndex(), const QModelIndex &bottomRight = QModelIndex());
 
 private:
-	void createBar(qreal x, qreal w, struct gasmix *gas);
+	void createBar(qreal x, qreal w, struct gasmix gas);
 	DivePlotDataModel *dataModel;
 	DiveCartesianAxis *hAxis;
 	struct dive diveCylinderStore;
