@@ -1,11 +1,66 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "qt-models/divelistmodel.h"
 #include "core/qthelper.h"
+#include "core/settings/qPrefGeneral.h"
 #include <QDateTime>
 
 DiveListSortModel::DiveListSortModel(QObject *parent) : QSortFilterProxyModel(parent)
 {
+	updateFilterState();
+}
 
+void DiveListSortModel::updateFilterState()
+{
+	if (filterString.isEmpty()) {
+		filteredRows.clear();
+		return;
+	}
+	// store this in local variables to avoid having to call these methods over and over
+	bool includeNotes = qPrefGeneral::filterFullTextNotes();
+	Qt::CaseSensitivity cs = qPrefGeneral::filterCaseSensitive() ? Qt::CaseSensitive : Qt::CaseInsensitive;
+
+	// get the underlying model and re-calculate the filter value for each dive
+	DiveListModel *mySourceModel = qobject_cast<DiveListModel *>(sourceModel());
+	filteredRows.clear();
+	filteredRows.resize(mySourceModel->rowCount());
+	for (int i = 0; i < mySourceModel->rowCount(); i++) {
+		QString fullText = includeNotes? mySourceModel->at(i)->fullText() : mySourceModel->at(i)->fullTextNoNotes();
+		filteredRows.at(i) = fullText.contains(filterString, cs);
+	}
+	updateDivesShownInTrips();
+}
+
+void DiveListSortModel::setSourceModel(QAbstractItemModel *sourceModel)
+{
+	QSortFilterProxyModel::setSourceModel(sourceModel);
+}
+void DiveListSortModel::setFilter(QString f)
+{
+	filterString = f;
+	updateFilterState();
+	invalidateFilter();
+	updateDivesShownInTrips();
+}
+
+void DiveListSortModel::resetFilter()
+{
+	filterString = "";
+	filteredRows.clear();
+	invalidateFilter();
+	updateDivesShownInTrips();
+}
+
+// filtering is way too slow on mobile. Maybe we should roll our own?
+bool DiveListSortModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
+{
+	Q_UNUSED(source_parent)
+
+	return filteredRows.size() > source_row ? filteredRows[source_row] : true;
+}
+
+int DiveListSortModel::shown()
+{
+	return rowCount();
 }
 
 int DiveListSortModel::getDiveId(int idx)
@@ -29,12 +84,32 @@ void DiveListSortModel::clear()
 {
 	DiveListModel *mySourceModel = qobject_cast<DiveListModel *>(sourceModel());
 	mySourceModel->clear();
+	filteredRows.clear();
 }
 
 void DiveListSortModel::addAllDives()
 {
 	DiveListModel *mySourceModel = qobject_cast<DiveListModel *>(sourceModel());
 	mySourceModel->addAllDives();
+	updateFilterState();
+}
+
+void DiveListSortModel::updateDivesShownInTrips()
+{
+	// if filtering is active, reset all the counts to zero, otherwise set them to the full count
+	struct dive_trip *dt = dive_trip_list;
+	int rc = rowCount();
+	while (dt) {
+		dt->showndives = rc ? 0 : dt->nrdives;
+		dt = dt->next;
+	}
+	for (int i = 0; i < rowCount(); i++) {
+		QVariant v = data(index(i, 0), DiveListModel::DiveRole);
+		DiveObjectHelper *d = v.value<DiveObjectHelper *>();
+		dt = d->getDive()->divetrip;
+		if (dt)
+			dt->showndives++;
+	}
 }
 
 DiveListModel *DiveListModel::m_instance = NULL;
@@ -151,6 +226,8 @@ QVariant DiveListModel::data(const QModelIndex &index, int role) const
 	switch(role) {
 	case DiveRole: return QVariant::fromValue<QObject*>(curr_dive);
 	case DiveDateRole: return (qlonglong)curr_dive->timestamp();
+	case FullTextRole: return curr_dive->fullText();
+	case FullTextNoNotesRole: return curr_dive->fullTextNoNotes();
 	}
 	return QVariant();
 
@@ -161,6 +238,8 @@ QHash<int, QByteArray> DiveListModel::roleNames() const
 	QHash<int, QByteArray> roles;
 	roles[DiveRole] = "dive";
 	roles[DiveDateRole] = "date";
+	roles[FullTextRole] = "fulltext";
+	roles[FullTextNoNotesRole] = "fulltextnonotes";
 	return roles;
 }
 
