@@ -5,7 +5,7 @@
 
 void TestDiveSiteDuplication::testReadV2()
 {
-	QCOMPARE(parse_file(SUBSURFACE_TEST_DATA "/dives/TwoTimesTwo.ssrf"), 0);
+	QCOMPARE(parse_file(SUBSURFACE_TEST_DATA "/dives/TwoTimesTwo.ssrf", &dive_table), 0);
 	QCOMPARE(dive_site_table.nr, 2);
 }
 
